@@ -1,6 +1,8 @@
 /*
-  basename
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  $NiH: basename.c,v 1.12 2002/09/16 12:42:28 dillo Exp $
+
+  basename.c -- return the last component of a pathname
+  Copyright (C) 1996-2002 Dieter Baron
 
   The author can be contacted at <dillo@giga.or.at>
 
@@ -26,7 +28,27 @@
 char *
 basename(char *name)
 {
-    char *p = strrchr(name, '/');
+    char *p;
+    int len;
 
-    return (p ? p+1 : name);
+    if (name == NULL || *name == '\0')
+	return ".";
+
+    len = strlen(name);
+
+    if (strspn(name, "/") == len)
+	return name + len-1;
+
+    while (name[len-1] == '/')
+	name[--len] = '\0';
+    
+    p = strrchr(name, '/');
+
+    if (p) {
+	if (*p == '\0')
+	    return p;
+	else
+	    return p+1;
+    }
+    return name;
 }
