@@ -1,6 +1,8 @@
 /*
-  direcotry -- handle directory cache
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  $NiH: directory.c,v 1.20 2002/09/16 12:42:29 dillo Exp $
+
+  directory.c -- handle directory cache
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -72,6 +74,58 @@ dir_free(directory *d)
 
 
 
+directory *
+dir_new(void)
+{
+    directory *dir;
+
+    if ((dir=malloc(sizeof(*dir))) == NULL)
+	return NULL;
+
+    dir->alloc_len = dir->len = dir->top = dir->cur = 0;
+    dir->size = sizeof(direntry);
+    dir->line = NULL;
+    dir->path = NULL;
+    dir->sorted = 0;
+
+    return dir;
+}
+
+
+
+int
+dir_add(directory *dir, direntry *e)
+{
+    int n;
+    direntry *line;
+    
+    if (dir->len >= dir->alloc_len) {
+	if (dir->alloc_len == 0)
+	    n = 16;
+	else if (dir->alloc_len < 1024)
+	    n = dir->alloc_len * 2;
+	else
+	    n = dir->alloc_len + 512;
+
+	if (dir->line == NULL)
+	    line = malloc(sizeof(direntry)*n);
+	else
+	    line = realloc(dir->line, sizeof(direntry)*n);
+	if (line == NULL)
+	    return -1;
+	dir->alloc_len = n;
+	dir->line = line;
+    }
+
+    n = dir->len++;
+    dir->line[n] = *e;
+    dir->line[n].pos = n;
+
+    return 0;    
+}
+
+
+
 directory *
 get_dir(char *path, int force)
 {
