@@ -2,8 +2,10 @@
 #define HAD_DIRECTORY_Y
 
 /*
+  $NiH: directory.h,v 1.13 2002/09/16 12:42:29 dillo Exp $
+
   directory.h -- handle directory cache
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -43,6 +45,7 @@ struct directory {
     int size;
     struct direntry *line;
     char *path;
+    int alloc_len;
     int sorted;
 };
 
@@ -50,11 +53,12 @@ typedef struct directory directory;
 
 
 
-void dir_free(directory *d);
-directory *get_dir(char *path, int force);
+int dir_add(directory *dir, direntry *entry);
 int dir_find(directory *dir, char *entry);
-
+void dir_free(directory *d);
+directory *dir_new(void);
 void dir_sort(directory *dir, int sort_type);
+directory *get_dir(char *path, int force);
 
 
 
