@@ -1,6 +1,8 @@
 /*
-  fn_basic -- bindable functions: basics
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  $NiH: fn_basic.c,v 1.28 2002/09/16 12:42:30 dillo Exp $
+
+  fn_basic.c -- bindable functions: basics
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -47,7 +49,7 @@ extern char version[];
 
 void fn_version(char **args)
 {
-	disp_status("%s", version);
+	disp_status(DISP_STATUS, "%s", version);
 }
 
 
@@ -70,31 +72,31 @@ fn_showname(char **args)
     if (args && strcmp(args[0], "-u") == 0) {
 	switch (binding_state) {
 	case bs_remote:
-	    disp_status("ftp://%s%s%s%s",
+	    disp_status(DISP_STATUS, "ftp://%s%s%s%s",
 			status.host, curdir->path,
 			(strcmp(curdir->path, "/") == 0 ? "" : "/"),
 			LIST_LINE(list, list->cur)->name);
 	    break;
 	case bs_tag:
-	    disp_status("ftp://%s%s", status.host,
+	    disp_status(DISP_STATUS, "ftp://%s%s", status.host,
 			LIST_LINE(list, list->cur)->name);
 	    break;
 	default:
-	    disp_status("");
+	    disp_status(DISP_STATUS, "");
 	}
     }
     else if (args && strcmp(args[0], "-l") == 0) {
 	switch (binding_state) {
 	case bs_remote:
 	    tmp = ((direntry *)LIST_LINE(list, list->cur))->link;
-	    disp_status("%s", tmp ? tmp : "");
+	    disp_status(DISP_STATUS, "%s", tmp ? tmp : "");
 	    break;
 	default:
-	    disp_status("");
+	    disp_status(DISP_STATUS, "");
 	}
     }
     else 
-	disp_status("%s", LIST_LINE(list, list->cur)->name);
+	disp_status(DISP_STATUS, "%s", LIST_LINE(list, list->cur)->name);
 }
 
 
@@ -116,14 +118,15 @@ void fn_help(char **args)
 	s = read_string("Function: ", 1);
 	if (s == NULL || s[0] == '\0') {
 	    free(s);
-	    disp_status("");
+	    disp_status(DISP_STATUS, "");
 	    return;
 	}
 	    
 	if ((i=find_function(s)) == -1)
-	    disp_status("no such function: %s", s);
+	    disp_status(DISP_STATUS, "no such function: %s", s);
 	else
-	    disp_status("%s: %s", functions[i].name, functions[i].help);
+	    disp_status(DISP_STATUS, "%s: %s",
+			functions[i].name, functions[i].help);
 
 	free(s);
 	break;
@@ -137,7 +140,7 @@ void fn_help(char **args)
 	}
 
 	disp_close(f, 1);
-	disp_status("");
+	disp_status(DISP_STATUS, "");
 	break;
 
     case 'k':
@@ -145,7 +148,7 @@ void fn_help(char **args)
 
 	b = get_function(c, bs_none);
 	if ((i=b->fn) == -1)
-	    disp_status("[%s%s] key is unbound",
+	    disp_status(DISP_STATUS, "[%s%s] key is unbound",
 			(binding_state != bs_none ?
 			 binding_statename[binding_state] : ""),
 			print_key(c, 0));
@@ -178,7 +181,7 @@ void fn_help(char **args)
 		    }
 	    }
 
-	    disp_status(buf);
+	    disp_status(DISP_STATUS, "%s", buf);
 	    free(buf);
 	}
 	break;
@@ -207,14 +210,14 @@ void fn_help(char **args)
 	}
 
 	disp_close(f, 1);
-	disp_status("");
+	disp_status(DISP_STATUS, "");
 	break;
 
     case 'o':
 	s = read_string("Option: ", 1);
 	if (s == NULL || s[0] == '\0') {
 	    free(s);
-	    disp_status("");
+	    disp_status(DISP_STATUS, "");
 	    return;
 	}
 	    
@@ -224,7 +227,7 @@ void fn_help(char **args)
 		break;
 	
 	if (option[i].name == NULL)
-	    disp_status("no such option: %s", s);
+	    disp_status(DISP_STATUS, "no such option: %s", s);
 	else {
 	    char *buf;
 	    
@@ -269,7 +272,7 @@ void fn_help(char **args)
 	    if (buf == NULL)
 		break;
 
-	    disp_status("%s (%s) [%s]: %s", 
+	    disp_status(DISP_STATUS, "%s (%s) [%s]: %s", 
 			option[i].name, option[i].shrt,
 			buf, option[i].help);
 	}
@@ -314,10 +317,10 @@ void fn_help(char **args)
 	}
 
 	disp_close(f, 1);
-	disp_status("");
+	disp_status(DISP_STATUS, "");
 	
     default:
-	disp_status("");
+	disp_status(DISP_STATUS, "");
     }
 }
 
@@ -352,7 +355,7 @@ void fn_lcd(char **args)
 
 	if (!rc_inrc) {
 	    lwd = getcwd(NULL, 1024);
-	    disp_status("Current local directory: %s", lwd);
+	    disp_status(DISP_STATUS, "Current local directory: %s", lwd);
 	    free(lwd);
 	}
 }
@@ -377,7 +380,7 @@ void fn_shell(char **args)
 	}
 	else
 		system("$SHELL -i");
-	disp_status("");
+	disp_status(DISP_STATUS, "");
 	reenter_disp();
 }
 
@@ -396,18 +399,18 @@ void fn_colon(char **args)
 	    line = p = read_string(": ", 1);
 	    if (line == NULL || line[0] == '\0') {
 		free(line);
-		disp_status("");
+		disp_status(DISP_STATUS, "");
 		return;
 	    }
 
 	    if ((cmd=rc_token(&p)) == NULL) {
-		disp_status("no function");
+		disp_status(DISP_STATUS, "no function");
 		return;
 	    }
 	}
 	
 	if ((i=find_function(cmd)) < 0) {
-	    disp_status("unknown function: %s", cmd);
+	    disp_status(DISP_STATUS, "unknown function: %s", cmd);
 	    if (line)
 		free(line);
 	    return;
@@ -417,7 +420,7 @@ void fn_colon(char **args)
 	    args = rc_list(p);
 	}
 
-	disp_status("");
+	disp_status(DISP_STATUS, "");
 	
 	if (functions[i].fn)
 	    functions[i].fn(args);
@@ -436,7 +439,7 @@ void fn_colon(char **args)
 
 void fn_deidle(char **args)
 {
-    ftp_noop();
+    ftp_deidle();
 }
 
 
@@ -449,7 +452,7 @@ void fn_site(char **args)
 	line = read_string("site ", 1);
 	if (line == NULL || line[0] == '\0') {
 	    free(line);
-	    disp_status("");
+	    disp_status(DISP_STATUS, "");
 	    return;
 	}
     }
@@ -476,7 +479,7 @@ void fn_response(char **args)
     FILE *f;
 
     if (ftp_history == NULL) {
-	disp_status("no exchange available");
+	disp_status(DISP_STATUS, "no exchange available");
 	return;
     }
     
@@ -543,14 +546,14 @@ void fn_set(char **args)
 	p = line = read_string("set ", 1);
 	opt = rc_token(&p);
 	if (opt == NULL) {
-	    disp_status("");
+	    disp_status(DISP_STATUS, "");
 	    return;
 	}
 	value = rc_token(&p);
     }
 
     if (opt == NULL || opt[0] == '\0') {
-	disp_status("no option.");
+	disp_status(DISP_STATUS, "no option.");
 	return;
     }
 
@@ -563,7 +566,7 @@ void fn_set(char **args)
 	if (rc_inrc)
 	    rc_error("unknown option: %s", opt);
 	else
-	    disp_status("unknown option: %s", opt);
+	    disp_status(DISP_STATUS, "unknown option: %s", opt);
 	return;
     }
 
@@ -578,7 +581,7 @@ void fn_set(char **args)
 	}
 	
 	if (!rc_inrc)
-	    disp_status("%s set to %d", option[i].name,
+	    disp_status(DISP_STATUS, "%s set to %d", option[i].name,
 			*(option[i].var.i));
 	break;
 
@@ -592,7 +595,7 @@ void fn_set(char **args)
 	}
 	
 	if (!rc_inrc)
-	    disp_status("%s set to `%c'", option[i].name,
+	    disp_status(DISP_STATUS, "%s set to `%c'", option[i].name,
 			*(option[i].var.i));
 	break;
 
@@ -605,7 +608,7 @@ void fn_set(char **args)
 	}
 	
 	if (!rc_inrc)
-	    disp_status("%s set to `%s'", option[i].name,
+	    disp_status(DISP_STATUS, "%s set to `%s'", option[i].name,
 			*(option[i].var.s));
 	break;
 	
@@ -625,7 +628,7 @@ void fn_set(char **args)
 	}
 
 	if (!rc_inrc)
-	    disp_status("%s set to %s", option[i].name,
+	    disp_status(DISP_STATUS, "%s set to %s", option[i].name,
 			(*(option[i].var.i) ? "on" : "off"));
 	break;
 
@@ -636,7 +639,7 @@ void fn_set(char **args)
 		    break;
 	    }
 	    if (option[i].values[ival] == NULL && !rc_inrc) {
-		disp_status("unknown value %s for option %s",
+		disp_status(DISP_STATUS, "unknown value %s for option %s",
 			    value, option[i].name);
 		break;
 	    }
@@ -648,7 +651,7 @@ void fn_set(char **args)
 	}
 
 	if (!rc_inrc)
-	    disp_status("%s set to %s", option[i].name,
+	    disp_status(DISP_STATUS, "%s set to %s", option[i].name,
 			option[i].values[(*(option[i].var.i))]);
 	break;
     }
@@ -714,7 +717,7 @@ void
 fn_leave_tag(char **args)
 {
     if (binding_state != bs_tag) {
-	disp_status("not in <tag> mode");
+	disp_status(DISP_STATUS, "not in <tag> mode");
 	return;
     }
 
@@ -731,7 +734,7 @@ fn_mkdir(char **args)
     if (args == NULL) {
 	name = read_string("Directory: ", 1);
 	if (name == NULL || name[0] == '\0') {
-	    disp_status("");
+	    disp_status(DISP_STATUS, "");
 	    free(name);
 	    return;
 	}
@@ -746,3 +749,30 @@ fn_mkdir(char **args)
 
     return;
 }
+
+
+
+
+void
+fn_rmdir(char **args)
+{
+    char *name;
+    
+    if (args == NULL) {
+	name = read_string("Directory: ", 1);
+	if (name == NULL || name[0] == '\0') {
+	    disp_status(DISP_STATUS, "");
+	    free(name);
+	    return;
+	}
+    }
+    else
+	name = args[0];
+
+    ftp_rmdir(name);
+
+    if (args == NULL)
+	free(name);
+
+    return;
+}
