@@ -1,6 +1,8 @@
 /*
-  fn_scroll -- bindable functions: scrolling
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  $NiH: fn_scroll.c,v 1.22 2002/09/16 12:42:31 dillo Exp $
+
+  fn_scroll.c -- bindable functions: scrolling
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -371,7 +373,7 @@ fn_isearch(char **args)
 	    aux_scroll(start-(win_lines/2), start, 0);
     }
 
-    disp_status("");
+    disp_status(DISP_STATUS, "");
     
     return;
 }
