@@ -1,6 +1,8 @@
 /*
-  fn_select -- bindable functions: selecting
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  $NiH: fn_select.c,v 1.27 2002/09/17 11:52:24 dillo Exp $
+
+  fn_select.c -- bindable functions: selecting
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -22,12 +24,23 @@
 
 
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <errno.h>
-#include <sys/types.h>
-#include <sys/stat.h>
+
+#include "config.h"
+
+#ifdef HAVE_BASENAME
+# ifdef HAVE_LIBGEN_H
+#  include <libgen.h>
+# endif
+#else
+char *basename(char *);
+#endif
+
 #include "directory.h"
 #include "bindings.h"
 #include "functions.h"
@@ -64,7 +77,8 @@ int
 aux_download(char *name, long size, int restart)
 {
     int err;
-    FILE *fin, *fout;
+    void *fin;
+    FILE *fout;
     struct stat st;
     char *mode;
     long start, rsize;
@@ -83,22 +97,22 @@ aux_download(char *name, long size, int restart)
     else
 	mode = "w";
     
-    if ((fout=fopen((char *)basename(name), mode)) == NULL) {
-	disp_status("can't %s `%s': %s",
+    if ((fout=fopen(basename(name), mode)) == NULL) {
+	disp_status(DISP_STATUS, "can't %s `%s': %s",
 		    (*mode == 'a' ? "append to" : "create"),
-		    (char *)basename(name), strerror(errno));
+		    basename(name), strerror(errno));
 	return -2;
     }
 
     if (size == -1)
 	size = rsize;
 
-    err = ftp_cat(fin, fout, start, size);
+    err = ftp_cat(fin, fout, start, size, 0);
 
     err |= ftp_fclose(fin);
     
     if (fclose(fout)) {
-	disp_status("error closing `%s': %s",
+	disp_status(DISP_STATUS, "error closing `%s': %s",
 		    basename(name), strerror(errno));
 	return -2;
     }
@@ -112,7 +126,8 @@ int
 aux_pipe(char *name, long size, int mode, char *cmd, int quietp)
 {
     int err;
-    FILE *fin, *fout;
+    void *fin;
+    FILE *fout;
 	
     if ((fin=ftp_retr(name, (mode ? mode : opt_mode), NULL, NULL)) == NULL)
 	return -2;
@@ -120,7 +135,7 @@ aux_pipe(char *name, long size, int mode, char *cmd, int quietp)
     if ((fout=disp_open(cmd, quietp)) == NULL)
 	return -2;
 
-    err = ftp_cat(fin, fout, 0, size);
+    err = ftp_cat(fin, fout, 0, size, 0);
 
     err |= ftp_fclose(fin);
 
@@ -137,14 +152,15 @@ aux_upload(char *name)
     int err;
     long size;
     struct stat st;
-    FILE *fin, *fout;
+    FILE *fin;
+    void *fout;
 
     if ((fin=fopen(name, "r")) == NULL) {
-	disp_status("can't open `%s': %s", name, strerror(errno));
+	disp_status(DISP_STATUS, "can't open `%s': %s", name, strerror(errno));
 	return -2;
     }
 
-    if ((fout=ftp_stor((char *)basename(name), opt_mode)) == NULL)
+    if ((fout=ftp_stor(basename(name), opt_mode)) == NULL)
 	return -2;
 
     if (stat(name, &st) >= 0)
@@ -152,12 +168,13 @@ aux_upload(char *name)
     else
 	size = -1;
     
-    err = ftp_cat(fin, fout, 0, size);
+    err = ftp_cat(fin, fout, 0, size, 1);
 
     err |= ftp_fclose(fout);
     
     if (fclose(fin)) {
-	disp_status("error closing `%s': %s", name, strerror(errno));
+	disp_status(DISP_STATUS, "error closing `%s': %s",
+		    name, strerror(errno));
 	return -2;
     }
 
@@ -178,6 +195,8 @@ fn_enter_get(char **args)
 	type = 'l';
 	size = -1;
     }
+    else if (curdir->len == 0)
+	return;
     else {
 	name = curdir->line[curdir->cur].name;
 	type = curdir->line[curdir->cur].type;
@@ -196,7 +215,7 @@ fn_enter_get(char **args)
 	    aux_download(name, size, 0);
 	break;
     default:
-	disp_status("Can't download special files.");
+	disp_status(DISP_STATUS, "Can't download special files.");
     }
 }
 
@@ -212,6 +231,8 @@ fn_enter_view(char **args)
 	name = args[0];
 	type = 'l';
     }
+    else if (curdir->len == 0)
+	return;
     else {
 	name = curdir->line[curdir->cur].name;
 	type = curdir->line[curdir->cur].type;
@@ -229,18 +250,22 @@ fn_enter_view(char **args)
 	    aux_view(name);
 	break;
     default:
-	disp_status("Can't view special files.");
+	disp_status(DISP_STATUS, "Can't view special files.");
     }
 }
 
 
 
-void fn_enter(char **args) { char *name; int type;
+void fn_enter(char **args)
+{
+    char *name; int type;
 
     if (args) {
 	name = args[0];
 	type = 'l';
     }
+    else if (curdir->len == 0)
+	return;
     else {
 	name = curdir->line[curdir->cur].name;
 	type = curdir->line[curdir->cur].type;
@@ -252,7 +277,7 @@ void fn_enter(char **args) { char *name; int type;
 	aux_enter(name);
 	break;
     default:
-	disp_status("Can enter only directories.");
+	disp_status(DISP_STATUS, "Can enter only directories.");
     }
 }
 
@@ -265,7 +290,10 @@ fn_reload(char **args)
     char *name;
     int sel;
 
-    name = strdup(curdir->line[curdir->cur].name);
+    if (curdir->len > 0)
+	name = strdup(curdir->line[curdir->cur].name);
+    else
+	name = NULL;
 
     d = ftp_cd(curdir->path, 1);
     if (d == NULL) {
@@ -276,7 +304,7 @@ fn_reload(char **args)
     curdir = NULL;
     change_curdir(d);
 
-    if ((sel=dir_find(curdir, name)) < 0)
+    if (name == NULL || (sel=dir_find(curdir, name)) < 0)
 	sel = 0;
     free(name);
     
@@ -297,6 +325,8 @@ fn_get(char **args)
 	type = 'l';
 	size = -1;
     }
+    else if (curdir->len == 0)
+	return;
     else {
 	name = curdir->line[curdir->cur].name;
 	type = curdir->line[curdir->cur].type;
@@ -309,7 +339,7 @@ fn_get(char **args)
 	aux_download(name, size, 0);
 	break;
     default:
-	disp_status("Can only download plain files.");
+	disp_status(DISP_STATUS, "Can only download plain files.");
     }
 }
 
@@ -325,6 +355,8 @@ fn_view(char **args)
 	name = args[0];
 	type = 'l';
     }
+    else if (curdir->len == 0)
+	return;
     else {
 	name = curdir->line[curdir->cur].name;
 	type = curdir->line[curdir->cur].type;
@@ -336,7 +368,7 @@ fn_view(char **args)
 	aux_view(name);
 	break;
     default:
-	disp_status("Can only view plain files.");
+	disp_status(DISP_STATUS, "Can only view plain files.");
     }
 }
 
@@ -360,6 +392,9 @@ fn_pipe(char **args)
 	    args++;
 	}
 	if (strcmp(args[0], ":") == 0) {
+	    if (curdir->len == 0)
+		return;
+
 	    name = curdir->line[curdir->cur].name;
 	    type = curdir->line[curdir->cur].type;
 	    size = curdir->line[curdir->cur].size;
@@ -406,7 +441,7 @@ fn_pipe(char **args)
 	if (freecmdp)
 	    free(cmd);
 	free(line);
-	disp_status("");
+	disp_status(DISP_STATUS, "");
 	return;
     }
 
@@ -416,7 +451,7 @@ fn_pipe(char **args)
 	aux_pipe(name, size, 0, cmd, quietp);
 	break;
     default:
-	disp_status("Can only view plain files.");
+	disp_status(DISP_STATUS, "Can only view plain files.");
     }
 }
 
@@ -462,7 +497,7 @@ fn_cd(char **args)
 	path = read_string("directory: ", 1);
 	
     if (path[0] == '\0') {
-	disp_status("");
+	disp_status(DISP_STATUS, "");
 	return;
     }
 
