@@ -1,6 +1,8 @@
  /*
-  fn_tag -- bindable functions: 
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  $NiH: fn_tag.c,v 1.29 2002/09/16 12:42:31 dillo Exp $
+
+  fn_tag.c -- bindable functions: 
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -20,18 +22,28 @@
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#include <config.h>
-
-#include <stdlib.h>
-#include <stdio.h>
-#include <errno.h>
-#include <string.h>
 #include <ctype.h>
+#include <errno.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "config.h"
+
 #ifdef HAVE_FNMATCH
 #include <fnmatch.h>
 #else
 #include <fnmatch_repl.h>
 #endif
+
+#ifdef HAVE_BASENAME
+# ifdef HAVE_LIBGEN_H
+#  include <libgen.h>
+# endif
+#else
+char *basename(char *);
+#endif
+
 #include "directory.h"
 #include "bindings.h"
 #include "functions.h"
@@ -42,6 +54,7 @@
 #include "util.h"
 #include "status.h"
 #include "list.h"
+#include "url.h"
 
 
 
@@ -54,7 +67,7 @@ fn_tag(char **args)
     int tagged, i;
 
     if (args) {
-	if (strcmp(args[0], "-t") == NULL || strcmp(args[0], "-u") == NULL) {
+	if (strcmp(args[0], "-t") == 0 || strcmp(args[0], "-u") == 0) {
 	    /* works in <remote> only */
 
 	    if (args[1] == NULL) {
@@ -86,7 +99,7 @@ fn_tag(char **args)
 	    size = -1;
 	    type = 'l';
 
-	    base = (char *)basename(file);
+	    base = basename(file);
 	    
 	    i = strlen(curdir->path);
 	    if ((base-file == 1 && i == 1)
@@ -116,13 +129,13 @@ fn_tag(char **args)
     }
 	
     if (tagged < -1)
-	disp_status("%d files untagged", -tagged);
+	disp_status(DISP_STATUS, "%d files untagged", -tagged);
     else if (tagged == -1)
-	disp_status("1 file untagged");
+	disp_status(DISP_STATUS, "1 file untagged");
     else if (tagged == 1)
-	disp_status("1 file tagged");
+	disp_status(DISP_STATUS, "1 file tagged");
     else if (tagged > 1)
-	disp_status("%d files tagged", tagged);
+	disp_status(DISP_STATUS, "%d files tagged", tagged);
 }
 
 
@@ -141,7 +154,7 @@ fn_cleartags(char **args)
 		list_reline(i);
 	}
 
-    disp_status("all tags cleared");
+    disp_status(DISP_STATUS, "all tags cleared");
 }
     
 
@@ -157,7 +170,7 @@ fn_gettags(char **args)
     int restart;
 
     if (!tag_anytags()) {
-	disp_status("no tags");
+	disp_status(DISP_STATUS, "no tags");
 	return;
     }
 
@@ -216,7 +229,7 @@ fn_savetags(char **args)
     save_as_url = 0;
     
     if (!tag_anytags()) {
-	disp_status("no tags");
+	disp_status(DISP_STATUS, "no tags");
 	return;
     }
 
@@ -230,13 +243,14 @@ fn_savetags(char **args)
     else {
 	name = read_string("File: ", 1);
 	if (name == NULL || name[0] == '\0') {
-	    disp_status("");
+	    disp_status(DISP_STATUS, "");
 	    return;
 	}
     }
 	    
     if ((f=fopen(name, "w")) == NULL) {
-	disp_status("can't create `%s': %s", name, strerror(errno));
+	disp_status(DISP_STATUS, "can't create `%s': %s",
+		    name, strerror(errno));
 	return;
     }
 
@@ -252,9 +266,10 @@ fn_savetags(char **args)
 			tags.line[i].name);
 
     if (ferror(f))
-	disp_status("write error on `%s': %s", name, strerror(errno));
+	disp_status(DISP_STATUS, "write error on `%s': %s",
+		    name, strerror(errno));
     else
-	disp_status("%d tag%s saved to `%s'%s", tags.len,
+	disp_status(DISP_STATUS, "%d tag%s saved to `%s'%s", tags.len,
 		    (tags.len == 1 ? "" : "s"), name,
 		    (save_as_url ? " (as URL)" : ""));
 
@@ -281,13 +296,14 @@ fn_loadtag(char **args)
     else {
 	fname = read_string("File: ", 1);
 	if (fname == NULL || fname[0] == '\0') {
-	    disp_status("");
+	    disp_status(DISP_STATUS, "");
 	    return;
 	}
     }
 
     if ((f=fopen(fname, "r")) == NULL) {
-	disp_status("can't open `%s': %s", fname, strerror(errno));
+	disp_status(DISP_STATUS, "can't open `%s': %s",
+		    fname, strerror(errno));
 	return;
     }
 
@@ -336,10 +352,11 @@ fn_loadtag(char **args)
     }
 
     if (ferror(f))
-	disp_status("read error: %s\n", strerror(errno));
+	disp_status(DISP_STATUS, "read error: %s\n", strerror(errno));
     else
-	disp_status("%d file%s tagged", count, (count == 1 ? "" : "s"));
-
+	disp_status(DISP_STATUS,
+		    (count == 1 ? "%d file tagged" : "%d files tagged"),
+		    count);
     fclose(f);
 }
 
@@ -349,11 +366,11 @@ void
 fn_saveurls(char **args)
 {
     FILE *f;
-    char *name;
-    int i;
+    char *name, *host, *p;
+    int i, n, len;
 
     if (!tag_anytags()) {
-	disp_status("no tags");
+	disp_status(DISP_STATUS, "no tags");
 	return;
     }
 
@@ -362,25 +379,41 @@ fn_saveurls(char **args)
     else {
 	name = read_string("File: ", 1);
 	if (name == NULL || name[0] == '\0') {
-	    disp_status("");
+	    disp_status(DISP_STATUS, "");
 	    return;
 	}
     }
 	    
     if ((f=fopen(name, "w")) == NULL) {
-	disp_status("can't create `%s': %s", name, strerror(errno));
+	disp_status(DISP_STATUS, "can't create `%s': %s",
+		    name, strerror(errno));
 	return;
     }
 
-    
+    host = mkhoststr(1, 1);
 
-    for (i=0; i<tags.len && !ferror(f); i++)
-	fprintf(f, "ftp://%s%s\n", status.host, tags.line[i].name);
+    n = 0;
+    p = NULL;
+    for (i=0; i<tags.len && !ferror(f); i++) {
+	len = url_enclen(tags.line[i].name, URL_UCHAR)+1;
+	if (len > n) {
+	    free(p);
+	    p = malloc(len);
+	    n = len;
+	}
+	fprintf(f, "ftp://%s%s", host, url_encode(p, tags.line[i].name,
+						  URL_XCHAR));
+    }
+    free(p);
+    free(host);
 
     if (ferror(f))
-	disp_status("write error on `%s': %s", name, strerror(errno));
+	disp_status(DISP_STATUS, "write error on `%s': %s",
+		    name, strerror(errno));
     else
-	disp_status("%d url%s saved", tags.len, (tags.len == 1 ? "" : "s"));
+	disp_status(DISP_STATUS,
+		    (tags.len == 1 ? "%d URL saved" : "%d URLs saved"),
+		    tags.len);
 
     fclose(f);
 
