@@ -1,6 +1,10 @@
 /*
    This file is automatically created from ``fntable.fn''; don't change
    this file, change ``fntable.fn'' instead.
+
+   Created from:
+    $NiH: mkfntab-c.m4,v 1.7 2002/09/16 12:42:38 dillo Exp $
+    $NiH: fntable.fn,v 1.22 2002/09/16 12:42:32 dillo Exp $
 */
 
 #include "directory.h"
@@ -23,6 +27,7 @@ function functions[] = {
   fn_reconnect , "reconnect", 0, "reconnect to server" ,
   fn_response , "response", 0, "display dialog with server" ,
   fn_mkdir , "mkdir", 0, "create directory on server" ,
+  fn_rmdir , "rmdir", 0, "remove directory on server" ,
   fn_prefix , "prefix", FN_PRE, "prefix digit" ,
   fn_bind , "bind", FN_RC, "bind key" ,
   fn_set , "set", FN_RC, "set option" ,
