@@ -4,6 +4,10 @@
 /*
    This file is automatically created from ``fntable.fn''; don't change
    this file, change ``fntable.fn'' instead.
+
+   Created from:
+    $NiH: mkfntab-h.m4,v 1.7 2002/09/16 12:42:38 dillo Exp $
+    $NiH: fntable.fn,v 1.22 2002/09/16 12:42:32 dillo Exp $
 */
 
 /* Basic Functions: fn_basic */
@@ -19,6 +23,7 @@ void fn_site(char **args);
 void fn_reconnect(char **args);
 void fn_response(char **args);
 void fn_mkdir(char **args);
+void fn_rmdir(char **args);
 void fn_prefix(char **args);
 void fn_bind(char **args);
 void fn_set(char **args);
