@@ -1,6 +1,8 @@
 /*
-  fputchar -- function version of putchar
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  $NiH: fputchar.c,v 1.12 2002/09/16 12:42:32 dillo Exp $
+
+  fputchar.c -- function version of putchar
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -24,6 +26,8 @@
 
 #include <stdio.h>
 
+#include "config.h"
+
 int
 fputchar(int c)
 {
