@@ -1,6 +1,8 @@
 /*
-  ftp -- ftp protocol functions
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  $NiH: ftp.c,v 1.74 2002/09/17 14:58:18 dillo Exp $
+
+  ftp.c -- ftp protocol functions
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -22,25 +24,34 @@
 
 
 
-#include "config.h"
-
-#include <stdio.h>
-#include <stdarg.h>
-#include <string.h>
-#include <stdlib.h>
-#include <ctype.h>
-#include <fcntl.h>
-#include <signal.h>
-#include <errno.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/time.h>
+#include <netinet/in.h>
+#include <ctype.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <unistd.h>
+#include <signal.h>
+#include <stdarg.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "config.h"
+
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
-#include <unistd.h>
-#include <fcntl.h>
-#include <netinet/in.h>
+
+#ifdef HAVE_BASENAME
+# ifdef HAVE_LIBGEN_H
+#  include <libgen.h>
+# endif
+#else
+char *basename(char *);
+#endif
+
 #include "directory.h"
 #include "display.h"
 #include "sockets.h"
@@ -52,17 +63,15 @@
 #include "status.h"
 #include "signals.h"
 
-extern char *prg;
-
 
 
 char *ftp_lcwd;
 char *ftp_pcwd;
 char ftp_curmode;		/* current transfer mode (' ' for unknown) */
-char ftp_anon;
 char *ftp_last_resp;
-char *ftp_host, *ftp_prt, *ftp_user, *ftp_pass;
+static char *_ftp_host, *_ftp_port, *_ftp_user, *_ftp_pass;
 				/* host, port, user, passwd (for reconnect) */
+static int _ftp_anon;		/* whether we're using anonymous ftp */
 
 struct ftp_hist *ftp_history;	/* command/response history */
 struct ftp_hist *ftp_hist_last; /* tail of history */
@@ -87,33 +96,43 @@ struct sockaddr *ftp_addr = (struct sockaddr *)&_ftp_addr;
 
 
 
-int ftp_put(char *fmt, ...);
-int ftp_abort(FILE *fin);
-int ftp_resp(void);
-void ftp_unresp(int resp);
-int ftp_port(void);
-FILE *ftp_accept(int fd, char *mode);
-int ftp_mode(char m);
-int ftp_cwd(char *path);
-int ftp_gethostaddr(int fd);
-void ftp_histf(char *fmt, ...);
-void ftp_hist(char *line);
+struct _ftp_transfer_stats {
+    long got, start, size;
+    int secs, stall_sec;
+    int ncur, ccur;
+    long *cur;
+};
 
-#ifdef ENABLE_TRANSFER_RATE
-static void _ftp_update_transfer(char *fmt, long got, long *cur,
-				 int old_sec, int new_sec);
-#endif
+
+
+static int ftp_abort(FILE *fin);
+static FILE *ftp_accept(int fd, char *mode);
+static int ftp_gethostaddr(int fd);
+static int ftp_mode(char m);
+static int ftp_port(void);
+static int ftp_put(char *fmt, ...);
+static int ftp_resp(void);
+static void ftp_unresp(int resp);
+
+static int _ftp_ascii2host(char *buf, char *buf2, int n, int *trail_cr);
+static int _ftp_host2ascii(char *buf, char *buf2, int n, int *trail_cr);
+
+static void _ftp_transfer_stats_init(struct _ftp_transfer_stats *tr,
+				     long start, long size, int ncur);
+static void _ftp_transfer_stats_cleanup(struct _ftp_transfer_stats *tr);
+static void _ftp_update_transfer(struct _ftp_transfer_stats *tr, long got,
+				 int secs);
 
 
 
 void
 ftp_init(void)
 {
-    ftp_pcwd = NULL;
+    ftp_lcwd = ftp_pcwd = NULL;
     ftp_curmode = ' ';
-    ftp_anon = 0;
+    _ftp_anon = 0;
     ftp_last_resp = NULL;
-    ftp_host = ftp_prt = ftp_pass = NULL;
+    _ftp_host = _ftp_user = _ftp_port = _ftp_pass = NULL;
     ftp_history = NULL;
     _ftp_keptresp = -1;
     ftp_dosnames = -1;
@@ -122,13 +141,24 @@ ftp_init(void)
     conin = conout = NULL;
 }
 
+
 
 int
-ftp_open(char *host, char *port)
+rftp_open(char *host, char *port, char *user, char *pass)
 {
     int fd;
-    
-    if ((fd=sopen(host, port, AF_UNSPEC)) == -1)
+
+    if (host) {
+	free(_ftp_host);
+	_ftp_host = strdup(host);
+	free(_ftp_port);
+	if (port && strcmp(port, "ftp") != 0)
+	    _ftp_port = strdup(port);
+	else
+	    _ftp_port = NULL;
+    }
+
+    if ((fd=sopen(_ftp_host, _ftp_port ? _ftp_port : "ftp", AF_UNSPEC)) == -1)
 	return -1;
     
     if (ftp_gethostaddr(fd) == -1) {
@@ -144,35 +174,57 @@ ftp_open(char *host, char *port)
 	return -1;
     }
     
+    if (ftp_resp() != 220) {
+	close(fd);
+	return -1;
+    }
+
+    ftp_remember_user(user, pass);
+
     return 0;
 }
 
 
 
 int
-ftp_login(char *host, char *user, char *pass)
+rftp_login(char *user, char *pass)
 {
     int resp;
+    char *b;
+    int free_pass;
 
-    if (ftp_resp() != 220)
-	return -1;
-
-    if (strcmp(user, "ftp") != 0 && strcmp(user, "anonymous") != 0) {
-	status.host = (char *)malloc(strlen(user)+strlen(host)+2);
-	sprintf(status.host, "%s@%s", user, host);
-	ftp_anon = 0;
-    }
-    else {
-	status.host = strdup(host);
-	ftp_anon = 1;
-    }
-
-    ftp_put("user %s", user);
+    status.host = mkhoststr(0, 0);
+    
+    ftp_put("user %s", _ftp_user);
     resp = ftp_resp();
 	
     if (resp == 331) {
+	free_pass = 0;
+	if (_ftp_pass)
+	    pass = _ftp_pass;
+	else {
+	    if (_ftp_anon) {
+		free_pass = 1;
+		pass = get_anon_passwd();
+	    }
+	    else {
+		b = (char *)malloc(strlen(status.host)+14);
+		sprintf(b, "Password (%s): ", status.host);
+		if (disp_active) {
+		    pass = read_string(b, 0);
+		    free_pass = 1;
+		}
+		else
+		    pass = getpass(b);
+		free(b);
+	    }
+	}
+	
 	ftp_put("pass %s", pass);
 	resp = ftp_resp();
+
+	if (free_pass)
+	    free(pass);
     }
 
     if (resp != 230)
@@ -188,7 +240,7 @@ ftp_login(char *host, char *user, char *pass)
 
 
 int
-ftp_site(char *cmd)
+rftp_site(char *cmd)
 {
     ftp_put("%s", cmd);
     return ftp_resp();
@@ -201,37 +253,21 @@ ftp_reconnect(void)
 {
     char *pass;
 
-    pass = ftp_pass;
+    pass = _ftp_pass;
 
-    if (ftp_host == NULL || ftp_prt == NULL || ftp_user == NULL)
+    if (_ftp_host == NULL || _ftp_port == NULL || _ftp_user == NULL)
 	return -1;
 
-    if (conin) {
-	fclose(conin);
-	conin = NULL;
-    }
-    if (conout) {
-	fclose(conout);
-	conout = NULL;
-    }
+    ftp_close();
 
-    if (pass == NULL) {
-	char *b;
-	
-	b = (char *)malloc(strlen(ftp_user)+strlen(ftp_host)+16);
-	sprintf(b, "Password (%s@%s): ", ftp_user, ftp_host);
-	pass = read_string(b, 0);
-	free(b);
-    }
+    disp_status(DISP_INFO, "connecting. . .");
 
-    disp_status("connecting. . .");
-
-    if (ftp_open(ftp_host, ftp_prt) == -1) {
+    if (ftp_open(NULL, NULL, NULL, NULL) == -1) {
 	/* Error printed in ftp_open or sopen. */
 	return -1;
     }
 
-    if (ftp_login(ftp_host, ftp_user, pass) == -1) {
+    if (ftp_login(NULL, NULL) == -1) {
 	/* ftp response is error message */
 	return -1;
     }
@@ -247,7 +283,7 @@ ftp_reconnect(void)
 
 
 int
-ftp_close(void)
+rftp_close(void)
 {
     int err = 0;
     
@@ -257,9 +293,13 @@ ftp_close(void)
     ftp_put("quit");
     if (ftp_resp() != 221)
 	err = 1;
+
+    if (conin)
+	fclose(conin);
+
+    if (conout)
+	fclose(conout);
     
-    fclose(conin);
-    fclose(conout);
     conin = conout = NULL;
     
     return err;
@@ -268,7 +308,7 @@ ftp_close(void)
 
 
 directory *
-ftp_list(char *path)
+rftp_list(char *path)
 {
     directory *dir;
     int fd;
@@ -277,10 +317,6 @@ ftp_list(char *path)
     if (ftp_mode('a') == -1 || ftp_cwd(path) == -1)
 	return NULL;
     
-    free(status.remote.path);
-    status.remote.path = strdup(path);
-    status_do(bs_remote);
-    
     if ((fd=ftp_port()) == -1)
 	return NULL;
     
@@ -340,16 +376,16 @@ ftp_cd(char *wd, int force)
 	
 
 
-FILE *
-ftp_retr(char *file, int mode, long *startp, long *sizep)
+void *
+rftp_retr(char *file, int mode, long *startp, long *sizep)
 {
     int fd;
     char *dir, *name, *can, *p;
     FILE *fin;
     
     can = canonical(file, NULL);
-    dir = dirname(can);
-    name = (char *)basename(can);
+    dir = xdirname(can);
+    name = basename(can);
     
     if (ftp_mode(mode) == -1 || ftp_cwd(dir) == -1)
 	return NULL;
@@ -374,6 +410,8 @@ ftp_retr(char *file, int mode, long *startp, long *sizep)
 	    if ((p=strrchr(ftp_last_resp, '(')) != NULL)
 		*sizep = strtol(p+1, NULL, 10);
 	}
+	else
+	    *sizep = -1;
     }
     if ((fin=ftp_accept(fd, "r")) == NULL) {
 	close(fd);
@@ -385,16 +423,16 @@ ftp_retr(char *file, int mode, long *startp, long *sizep)
 
 
 
-FILE *
-ftp_stor(char *file, int mode)
+void *
+rftp_stor(char *file, int mode)
 {
     int fd, resp;
     char *dir, *name, *can;
     FILE *fin;
     
     can = canonical(file, NULL);
-    dir = dirname(can);
-    name = (char *)basename(can);
+    dir = xdirname(can);
+    name = basename(can);
     
     if (ftp_mode(mode) == -1 || ftp_cwd(dir) == -1)
 	return NULL;
@@ -418,7 +456,7 @@ ftp_stor(char *file, int mode)
 
 
 int
-ftp_fclose(FILE *f)
+rftp_fclose(void *f)
 {
     int err;
 
@@ -433,7 +471,7 @@ ftp_fclose(FILE *f)
 
 
 int
-ftp_mkdir(char *path)
+rftp_mkdir(char *path)
 {
     ftp_put("mkd %s", path);
     if (ftp_resp() != 257)
@@ -445,7 +483,19 @@ ftp_mkdir(char *path)
 
 
 int
-ftp_noop(void)
+rftp_rmdir(char *path)
+{
+    ftp_put("rmd %s", path);
+    if (ftp_resp() != 250)
+	return -1;
+    
+    return 0;
+}
+
+
+
+int
+rftp_deidle(void)
 {
     ftp_put("noop");
     if (ftp_resp() != 200)
@@ -457,7 +507,7 @@ ftp_noop(void)
 
 
 char *
-ftp_pwd(void)
+rftp_pwd(void)
 {
     char *s, *e, *dir;
 
@@ -508,7 +558,7 @@ char *
 ftp_gets(FILE *f)
 {
     char buf[8192], *line;
-    int l;
+    int l, l2;
     
     if (fgets(buf, 8192, f) == NULL)
 	return NULL;
@@ -521,12 +571,13 @@ ftp_gets(FILE *f)
 	    free(line);
 	    return NULL;
 	}
-	l += strlen(buf);
-	if ((line=realloc(line, l+1)) == NULL) {
-	    disp_status("malloc failure");
+	l2 = strlen(buf);
+	if ((line=realloc(line, l+l2+1)) == NULL) {
+	    disp_status(DISP_ERROR, "malloc failure");
 	    return NULL;
 	}
 	strcpy(line+l, buf);
+	l += l2;
     }
     if (line[l-2] == '\r')
 	line[l-2] = '\0';
@@ -538,14 +589,14 @@ ftp_gets(FILE *f)
 
 
 
-int
+static int
 ftp_put(char *fmt, ...)
 {
     char buf[8192];
     va_list argp;
     
     if (conout == NULL) {
-	disp_status("not connected");
+	disp_status(DISP_ERROR, "not connected");
 	return -1;
     }
     
@@ -553,18 +604,15 @@ ftp_put(char *fmt, ...)
     vsprintf(buf, fmt, argp);
     va_end(argp);
     
-    if (strncmp(buf, "pass ", 5) == 0 && ftp_anon == 0) {
-	disp_status("-> pass ********");
-	ftp_hist(strdup("-> pass ********"));
-    }
-    else {
-	disp_status("-> %s", buf);
-	ftp_histf("-> %s", buf);
-    }
+    if (strncmp(buf, "pass ", 5) == 0 && _ftp_anon == 0)
+	disp_status(DISP_PROTO, "-> pass ********");
+    else
+	disp_status(DISP_PROTO, "-> %s", buf);
     fprintf(conout, "%s\r\n", buf);
     
     if (fflush(conout) || ferror(conout)) {
-	disp_status("error writing to server: %s", strerror(errno));
+	disp_status(DISP_ERROR, "error writing to server: %s",
+		    strerror(errno));
 	return -1;
     }
     
@@ -573,7 +621,7 @@ ftp_put(char *fmt, ...)
 
 
 
-int
+static int
 ftp_abort(FILE *fin)
 {
     int resp;
@@ -590,18 +638,24 @@ ftp_abort(FILE *fin)
 	return 0;
     
     /* do abort */
-    disp_status("-> <attention>");
-    ftp_hist(strdup("-> <attention>"));
+    disp_status(DISP_PROTO, "-> <attention>");
 	     
     if (send(fileno(conout), "\377\364\377" /* IAC IP IAC */, 3,
-	     MSG_OOB) != 3)
+	     MSG_OOB) != 3) {
+	disp_status(DISP_ERROR, "cannot send attention: %s",
+		    strerror(errno));
 	return -1;
+    }
 
-    if (fputc('\362' /* DM */, conout) == EOF)
+    if (fputc('\362' /* DM */, conout) == EOF) {
+	disp_status(DISP_ERROR, "cannot send attention: %s",
+		    strerror(errno));
 	return -1;
+    }
 
     ftp_put("abor");
 
+    /* XXX: what about uploads? */
     /* read remaining bytes from data connection */
     while (fread(buf, 4096, 1, fin) > 0)
 	;
@@ -626,7 +680,7 @@ ftp_abort(FILE *fin)
     else if (resp == 426) {
 	resp = ftp_resp();
     	ftp_unresp(426);
-	disp_status("426 Transfer aborted.");
+	disp_status(DISP_STATUS, "426 Transfer aborted.");
 	    
 	return resp == 226;
     }
@@ -637,7 +691,7 @@ ftp_abort(FILE *fin)
 
 
 
-int
+static int
 ftp_resp(void)
 {
     char *line;
@@ -650,49 +704,53 @@ ftp_resp(void)
     }
 
     if (conin == NULL) {
-	disp_status("not connectecd");
+	disp_status(DISP_ERROR, "not connected");
 	return -1;
     }
 
     clearerr(conin);
     if ((line=ftp_gets(conin)) == NULL) {
 	if (ferror(conin))
-	    disp_status("read error from server: %s", strerror(errno));
+	    disp_status(DISP_ERROR, "read error from server: %s",
+			strerror(errno));
 	else
-	    disp_status("connection to server lost");
+	    disp_status(DISP_ERROR, "connection to server lost");
 	return -1;
     }
 
+    /* XXX: DISP_PROTO should be used for first line */
     resp = atoi(line);
-    disp_status("%s", line);
+    disp_status(DISP_STATUS, "%s", line);
     free(ftp_last_resp);
     ftp_last_resp = strdup(line);
 
     while (!(isdigit(line[0]) && isdigit(line[1]) &&
 	     isdigit(line[2]) && line[3] == ' ')) {
-	ftp_hist(line);
+	disp_status(DISP_HIST, "%s", line);
 	
 	if ((line=ftp_gets(conin)) == NULL) {
-	    disp_status("read error from server: %s", strerror(errno));
+	    disp_status(DISP_ERROR, "read error from server: %s",
+			strerror(errno));
 	    return -1;
 	}
     }
 
-    ftp_hist(line);
+    disp_status(DISP_HIST, "%s", line);
     
     return resp;
 }
 
 
 
-void ftp_unresp(int resp)
+static void
+ftp_unresp(int resp)
 {
     _ftp_keptresp = resp;
 }
 
 
 
-int
+static int
 ftp_port(void)
 {
     int fd, port, val, i, delim;
@@ -792,7 +850,7 @@ ftp_port(void)
 	    strncpy(bport, s, e-s);
 	    bport[e-s] = '\0';
 	    sport = bport;
-	    saddr = ftp_host;
+	    saddr = _ftp_host;
 	}
 	fd = sopen(saddr, sport, ftp_addr->sa_family);
     }
@@ -802,7 +860,7 @@ ftp_port(void)
 
 
 
-FILE *
+static FILE *
 ftp_accept(int fd, char *mode)
 {
     int len, ns;
@@ -824,7 +882,7 @@ ftp_accept(int fd, char *mode)
 
 
 
-int
+static int
 ftp_mode(char m)
 {
     if (m == ftp_curmode)
@@ -841,7 +899,7 @@ ftp_mode(char m)
 
 
 int
-ftp_cwd(char *path)
+rftp_cwd(char *path)
 {
     char *s, *e;
     int off;
@@ -887,181 +945,247 @@ ftp_cwd(char *path)
 
 
 int
-ftp_cat(FILE *fin, FILE *fout, long start, long size)
+ftp_cat(void *fin, void *fout, long start, long size, int upload)
 {
-    char buf[4096], fmt[4096];
-    int c, n, err;
-    long got;
-#ifdef ENABLE_TRANSFER_RATE
-    struct itimerval itv;
-    long cur[4];
+    char buf[4096], buf2[8192], *p;
+    int nread, nwritten, err, trail_cr, errno_copy;
+    enum { ERR_NONE, ERR_FIN, ERR_FOUT } error_cause;
     int old_alarm;
-#else
-    time_t oldt, newt;
-#endif
-
-    if (size >= 0) {
-#ifdef ENABLE_TRANSFER_RATE
-	sprintf(fmt, "transferred %%ld/%ld "
-		"(total: %%.2fkb/s, current: %%.2fkb/s)",
-		size);
-#else
-	sprintf(fmt, "transferred %%ld/%ld", size);
-#endif
-    }
-    else {
-#ifdef ENABLE_TRANSFER_RATE
-	strcpy(fmt, "transferred %ld (total: %%.2fkb/s, current: %%.2fkb/s)");
-#else
-	strcpy(fmt, "transferred %ld");
-#endif
-    }
+    long got;
+    struct itimerval itv;
+    struct _ftp_transfer_stats trstat;
+    int do_read;
+    int (*fn_read)(void *, size_t, void *);
+    int (*fn_write)(void *, size_t, void *);
+    int (*fn_eof)(void *);
 
     got = start;
     signal(SIGINT, sig_remember);
 
-#ifdef ENABLE_TRANSFER_RATE
-    cur[0] = cur[1] = cur[2] = cur[3] = 0;
+    _ftp_transfer_stats_init(&trstat, start, size, 3);
     old_alarm = sig_alarm = sig_intr = 0;
     itv.it_value.tv_sec = itv.it_interval.tv_sec = 1;
     itv.it_value.tv_usec = itv.it_interval.tv_usec = 0;
     setitimer(ITIMER_REAL, &itv, NULL);
-#else
-    oldt = 0;
-#endif
 
-    if (ftp_curmode == 'i')
-	while ((n=fread(buf, 1, 4096, fin)) > 0) {
-	    if (sig_intr)
-		break;
-	    if (fwrite(buf, 1, n, fout) != n) {
-		break;
-	    }
-	    got += n;
-#ifdef ENABLE_TRANSFER_RATE
-	    if (old_alarm != sig_alarm) {
-		_ftp_update_transfer(fmt, got-start, cur,
-				     old_alarm, sig_alarm);
-		old_alarm = sig_alarm;
-	    }
-#else
-	    if ((newt=time(NULL)) != oldt) {
-		disp_status(fmt, got);
-		oldt = newt;
-	    }
-#endif
-	}
-    else
-	while ((c=getc(fin)) != EOF) {
-	    if (sig_intr)
-		break;
-	    got++;
-	    if (c == '\r') {
-		if ((c=getc(fin)) != '\n')
-		    putc('\r', fout);
-		ungetc(c, fin);
-	    }
-	    else {
-		putc(c, fout);
-	    }
-	    if (ferror(fout))
-		break;
+    /* XXX: error check */
+    if (upload) {
+	/* fout is to server */
+	if (ftp_xfer_start(fout) < 0)
+	    return -1;
 
-#ifdef ENABLE_TRANSFER_RATE
-	    if (old_alarm != sig_alarm) {
-		_ftp_update_transfer(fmt, got-start, cur,
-				     old_alarm, sig_alarm);
-		old_alarm = sig_alarm;
+	fn_read = rftp_xfer_read;
+	fn_eof = rftp_xfer_eof;
+	fn_write = ftp_xfer_write;
+
+	rftp_xfer_start(fin);
+    }
+    else {
+	/* fin is from server */
+
+	if (ftp_xfer_start(fin) < 0)
+	    return -1;
+	   
+	fn_read = ftp_xfer_read;
+	fn_eof = ftp_xfer_eof;
+	fn_write = rftp_xfer_write;
+
+	rftp_xfer_start(fout);
+    }
+
+    error_cause = ERR_NONE;
+    trail_cr = 0;
+    do_read = 1;
+    for (;;) {
+	if (do_read) {
+	    if ((nread=fn_read(buf, 4096, fin)) > 0) {
+		do_read = 0;
+		nwritten = 0;
+		if (ftp_curmode != 'a')
+		    p = buf;
+		else {
+		    p = buf2;
+		    if (upload)
+			nread = _ftp_host2ascii(buf, buf2, nread, &trail_cr);
+		    else
+			nread = _ftp_ascii2host(buf, buf2, nread, &trail_cr);
+		}
+	    }
+	    else if (nread < 0) {
+		if (!fn_eof(fin)) {
+		    errno_copy = errno;
+		    error_cause = ERR_FIN;
+		}
+		break;
 	    }
-#else
-	    if (got%512 == 0 && (newt=time(NULL)) != oldt) {
-		disp_status(fmt, got);
-		oldt = newt;
-	    } 
-#endif
 	}
+	else {
+	    if ((err=fn_write(p+nwritten, nread-nwritten, fout)) < 0) {
+		errno_copy = errno;
+		error_cause = ERR_FOUT;
+		break;
+	    }
+	    nwritten += err;
+	    got += err;
+	    
+	    if (nwritten == nread)
+		do_read = 1;
+	}
+	
+	if (old_alarm != sig_alarm) {
+	    _ftp_update_transfer(&trstat, got, sig_alarm);
+	    old_alarm = sig_alarm;
+	}
+
+	if (sig_intr)
+	    break;
+    }
 
     signal(SIGINT, sig_end);
-#ifdef ENABLE_TRANSFER_RATE
     itv.it_value.tv_sec = itv.it_interval.tv_sec = 0;
     itv.it_value.tv_usec = itv.it_interval.tv_usec = 0;
     setitimer(ITIMER_REAL, &itv, NULL);
-#endif
 
-    if (ferror(fin) || sig_intr) {
-	errno = 0;
+    _ftp_transfer_stats_cleanup(&trstat);
+
+    if (error_cause || sig_intr) {
 	sig_intr = 0;
-	ftp_abort(fin);
-	return -1;
-    }
-    else if (ferror(fout)) {
-	err = errno;
-	errno = 0;
-	ftp_abort(fin);
-	disp_status("write error: %s", strerror(err));
+	ftp_xfer_stop((upload ? fout : fin), 1);
+	/* this is a hack, we should really use local file methods */
+	rftp_xfer_stop((upload ? fin : fout), 0);
+	switch (error_cause) {
+	case ERR_FIN:
+	    disp_status(DISP_ERROR, "read error: %s", strerror(errno_copy));
+	    break;
+	case ERR_FOUT:
+	    disp_status(DISP_ERROR, "write error: %s", strerror(errno_copy));
+	    break;
+	default:
+	    ;
+	}
 	return -1;
     }
 
+    ftp_xfer_stop((upload ? fout : fin), 0);
+    rftp_xfer_stop((upload ? fin : fout), 0);
     return 0;
 }
 
 
 
-#ifdef ENABLE_TRANSFER_RATE
 static void
-_ftp_update_transfer(char *fmt, long got, long *cur, int old_sec, int new_sec)
+_ftp_update_transfer(struct _ftp_transfer_stats *tr, long got, int secs)
 {
-    long step;
-    int nsec;
+    long base, step;
+    int nsec, i, stsec, eta;
+    double rtot, rcur, rbps;
+    char b[512];
 
-    nsec = new_sec-old_sec;
+    nsec = secs-tr->secs;
 
-    switch (nsec) {
-    case 0:
-	break;
-
-    case 1:
-	cur[0] = cur[1];
-	cur[1] = cur[2];
-	cur[2] = cur[3];
-	break;
-
-    case 2:
-	cur[0] = cur[2];
-	cur[1] = cur[3];
-	cur[2] = cur[1] + (got-cur[1])/2;
-	break;
-
-    default:
-	step = (got-cur[3]) / nsec;
-	cur[0] = cur[3]+step*(nsec-3);
-	cur[1] = cur[3]+step*(nsec-2);
-	cur[2] = cur[3]+step*(nsec-1);
-	break;
+    if (nsec) {
+	base = tr->cur[tr->ccur];
+	step = (got-base) / nsec;
+	for (i=0; i<nsec-1; i++) {
+	    tr->ccur = (tr->ccur+1) % tr->ncur;
+	    tr->cur[tr->ccur] = base + step*i;
+	}
+	tr->ccur = (tr->ccur+1) % tr->ncur;
+	tr->cur[tr->ccur] = got;
     }
-    cur[3] = got;
+    if (tr->got != got) {
+	tr->got = got;
+	tr->stall_sec = secs;
+    }
+    tr->secs = secs;
 
-    disp_status(fmt, got, got/(float)(new_sec*1024),
-		(cur[3]-cur[0])/((float)3*1024));
+    rtot = (got-tr->start)/(double)(secs*1024);
+    rcur = ((tr->cur[tr->ccur] - tr->cur[(tr->ccur+1)%tr->ncur])
+	    / ((double)(secs < tr->ncur-1 ? secs : tr->ncur-1)*1024));
+    if (got == tr->start)
+	eta = -1;
+    else {
+	rbps = (got-tr->start)/(double)secs;
+	eta = (tr->size-got) / rbps;
+    }
+
+    stsec = secs - tr->stall_sec;
+    if (stsec > opt_stall) {
+	disp_status(DISP_ERROR, "stalled for more than %d seconds, aborting.");
+	sig_intr = 1;
+    }
+    else {
+	if (stsec > tr->ncur) {
+	    if (stsec >= 60)
+		sprintf(b, "stalled: %d:%02ds", stsec/60, stsec%60);
+	    else
+		sprintf(b, "stalled: %ds", stsec);
+	}
+	else
+	    sprintf(b, "cur: %.2fkb/s", rcur);
+	    
+	if (eta != -1) {
+	    if (eta > 60*60)
+		sprintf(b+strlen(b), ", eta: %d:%02d:%02ds",
+			eta/(60*60), (eta/60)%60, eta%60);
+	    else if (eta > 60)
+		sprintf(b+strlen(b), ", eta: %d:%02ds",
+			eta/60, eta%60);
+	    else
+		sprintf(b+strlen(b), ", eta: %ds", eta);
+	}
+
+	if (tr->size != -1)
+	    disp_status(DISP_STATUS,
+			"transferred %ld/%ld (tot: %.2fkb/s, %s)",
+			got, tr->size, rtot, b);
+	else
+	    disp_status(DISP_STATUS,
+			"transferred %ld (tot: %.2fkb/s, %s)",
+			got, rtot, b);
+    }
 }
-#endif /* ENABLE_TRANSFER_RATE */
 
 
 
-int
+static void
+_ftp_transfer_stats_init(struct _ftp_transfer_stats *tr,
+			 long start, long size, int cur_secs)
+{
+    int i;
+
+    /* XXX: check return value */
+    tr->ncur = cur_secs+1;
+    tr->cur = malloc(tr->ncur*sizeof(long));
+    tr->ccur = 0;
+    for (i=0; i<tr->ncur; i++)
+	tr->cur[i] = start;
+
+    tr->got = tr->start = start;
+    tr->size = size;
+    tr->secs = tr->stall_sec = 0;
+}
+
+
+
+static void
+_ftp_transfer_stats_cleanup(struct _ftp_transfer_stats *tr)
+{
+    free(tr->cur);
+    tr->ncur = 0;
+}
+
+
+
+static int
 ftp_gethostaddr(int fd)
 {
     int len;
 	
     len = sizeof(_ftp_addr);
     if (getsockname(fd, ftp_addr, &len) == -1) {
-	if (disp_active)
-	    disp_status("can't get host address: %s",
-			strerror(errno));
-	else
-	    fprintf(stderr, "%s: can't get host addres: %s\n",
-		    prg, strerror(errno));
-	
+	disp_status(DISP_ERROR, "can't get host address: %s",
+		    strerror(errno));
 	return -1;
     }
     
@@ -1070,25 +1194,6 @@ ftp_gethostaddr(int fd)
 
 
 
-void
-ftp_histf(char *fmt, ...)
-{
-    char buf[1024];
-    
-    va_list argp;
-
-    if (ftp_hist_size == 0)
-	return;
-
-    va_start(argp, fmt);
-    vsprintf(buf, fmt, argp);
-    va_end(argp);
-
-    ftp_hist(strdup(buf));
-}
-
-
-
 void ftp_hist(char *line)
 {
     struct ftp_hist *p;
@@ -1165,3 +1270,247 @@ opts_mode(int c, int *cp)
 	opt_mode = 'i';
     }
 }
+
+
+
+char *
+ftp_host(void)
+{
+    return _ftp_host;
+}
+
+
+
+char *
+ftp_prt(void)
+{
+    return _ftp_port;
+}
+
+
+
+char *
+ftp_user(void)
+{
+    return _ftp_user;
+}
+
+
+
+char *
+ftp_pass(void)
+{
+    return _ftp_pass;
+}
+
+
+
+int
+ftp_anon(void)
+{
+    return _ftp_anon;
+}
+
+
+
+static int
+_ftp_ascii2host(char *buf, char *buf2, int n, int *trail_cr)
+{
+    char *s, *t;
+    int cr;
+
+    if (n == 0)
+	return 0;
+
+    s = buf;
+    t = buf2;
+    cr = *trail_cr;
+
+    for (; n; --n,s++) {
+	if (cr) {
+	    if (*s != '\n')
+		*t++ = '\r';
+	    cr = 0;
+	}
+	if (*s == '\r')
+	    cr = 1;
+	else
+	    *(t++) = *s;
+    }
+
+    *trail_cr = cr;
+    
+    return t-buf2;
+}
+
+
+
+static int
+_ftp_host2ascii(char *buf, char *buf2, int n, int *trail_cr)
+{
+    char *s, *t;
+    int cr;
+
+    if (n == 0)
+	return 0;
+
+    s = buf;
+    t = buf2;
+    cr = *trail_cr;
+
+    for (; n; --n,s++) {
+	if (*s == '\n' && !cr)
+	    *t++ = '\r';
+	cr = (*s == '\r');
+	*t++ = *s;
+    }
+
+    *trail_cr = cr;
+    
+    return t-buf2;
+}
+
+
+
+void
+ftp_remember_user(char *user, char *pass)
+{
+    if (user) {
+	free(_ftp_user);
+	_ftp_user = strdup(user);
+	
+	free(_ftp_pass);
+	if (pass)
+	    _ftp_pass = strdup(pass);
+	else
+	    _ftp_pass = NULL;
+
+	if (strcmp(_ftp_user, "ftp") != 0
+	    && strcmp(_ftp_user, "anonymous") != 0)
+	    _ftp_anon = 0;
+	else
+	    _ftp_anon = 1;
+    }
+}
+
+
+
+void
+ftp_remember_host(char *host, char *port)
+{
+    if (host) {
+	free(_ftp_host);
+	_ftp_host = strdup(host);
+
+	free(_ftp_port);
+	if (port)
+	    _ftp_port = strdup(port);
+	else
+	    _ftp_port = NULL;
+    }
+}
+
+
+
+/* this method is also used for local files */
+int
+rftp_xfer_read(void *buf, size_t len, void *file)
+{
+    int n;
+    FILE *f;
+    fd_set fdset;
+
+    f = (FILE *)file;
+    
+    for (;;) {
+	n = fread(buf, 1, len, f);
+
+	if (n == 0) {
+	    if (ferror(f) && (errno == EINTR || errno == EAGAIN)) {
+		clearerr(f);
+		if (errno == EAGAIN) {
+		    FD_ZERO(&fdset);
+		    FD_SET(fileno(f), &fdset);
+		    
+		    if (select(fileno(f)+1, &fdset, NULL, NULL, NULL) == -1
+			|| !FD_ISSET(fileno(f), &fdset))
+			return 0;
+		    continue;
+		}
+	    }
+	    else
+		return -1;
+	}
+
+	return n;
+    }
+}
+
+
+
+int
+rftp_xfer_start(void *file)
+{
+    FILE *f;
+
+    f = (FILE *)file;
+
+    set_file_blocking(fileno(f), 0);
+
+    return 0;
+}
+
+
+
+int
+rftp_xfer_stop(void *file, int aborting)
+{
+    if (aborting)
+	return ftp_abort((FILE *)file);
+    
+    return 0;
+}
+
+
+
+/* this method is also used for local files */
+int
+rftp_xfer_write(void *buf, size_t len, void *file)
+{
+    int n;
+    FILE *f;
+    fd_set fdset;
+
+    f = (FILE *)file;
+
+    for (;;) {
+	n = fwrite(buf, 1, len, f);
+
+	if (n == 0) {
+	    if (ferror(f) && (errno == EINTR || errno == EAGAIN)) {
+		clearerr(f);
+		if (errno == EAGAIN) {
+		    FD_ZERO(&fdset);
+		    FD_SET(fileno(f), &fdset);
+		    
+		    if (select(fileno(f)+1, NULL, &fdset, NULL, NULL) == -1
+			|| !FD_ISSET(fileno(f), &fdset))
+			return 0;
+		    continue;
+		}
+	    }
+	    else
+		return -1;
+	}
+	return n;
+    }
+}
+
+
+
+/* this method is also used for local files */
+int
+rftp_xfer_eof(void *file)
+{
+    return feof((FILE *)file);
+}
