@@ -2,8 +2,10 @@
 #define HAD_FTP_H
 
 /*
+  $NiH: ftp.h,v 1.24 2002/09/16 12:42:33 dillo Exp $
+
   ftp.h -- ftp protocol functions
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -27,6 +29,8 @@
 
 #include <stdio.h>
 
+#include "methods.h"
+
 struct ftp_hist {
     char *line;
     struct ftp_hist *next;
@@ -34,28 +38,26 @@ struct ftp_hist {
 
 extern struct ftp_hist *ftp_history;
 
-extern char ftp_anon;
-extern char *ftp_host, *ftp_prt, *ftp_user, *ftp_pass;
 extern char **ftp_response;
+extern char *ftp_lcwd;
 extern char *ftp_pcwd;
 
 
 
-void ftp_init(void);
-int ftp_open(char *host, char *port);
-int ftp_login(char *host, char *user, char *pass);
-int ftp_reconnect(void);
-int ftp_close(void);
-directory *ftp_list(char *path);
+int ftp_anon(void);
+int ftp_cat(void *fin, void *fout, long start, long size, int upload);
 directory *ftp_cd(char *wd, int force);
-FILE *ftp_retr(char *file, int mode, long *startp, long *sizep);
-FILE *ftp_stor(char *file, int mode);
-int ftp_fclose(FILE *f);
-int ftp_mkdir(char *path);
-int ftp_site(char *cmd);
-int ftp_noop(void);
-char *ftp_pwd(void);
 char *ftp_gets(FILE *f);
-int ftp_cat(FILE *fin, FILE *fout, long start, long size);
+void ftp_hist(char *line);
+char *ftp_host(void);
+void ftp_init(void);
+char *ftp_pass(void);
+char *ftp_prt(void);
+int ftp_reconnect(void);
+void ftp_remember_host(char *host, char *port);
+void ftp_remember_user(char *user, char *pass);
+char *ftp_user(void);
+
+
 
 #endif /* ftp.h */
