@@ -1,6 +1,8 @@
 /*
-  functions -- auxiliary functions for bindable function handling
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  $NiH: functions.c,v 1.13 2002/09/16 12:42:34 dillo Exp $
+
+  functions.c -- auxiliary functions for bindable function handling
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -98,7 +100,7 @@ void
 show_prefix(void)
 {
 
-	disp_status(": %d", prefix_arg);
+	disp_status(DISP_STATUS, ": %d", prefix_arg);
 }
 
 
