@@ -2,8 +2,10 @@
 #define HAD_FUNCTIONS_H
 
 /*
+  $NiH: functions.h,v 1.12 2002/09/17 11:59:46 dillo Exp $
+
   functions.h -- auxiliary functions for bindable function handling
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -26,7 +28,7 @@
 
 
 struct function {
-	void (*fn)();
+	void (*fn)(char **args);
 	char *name;
 	int type;
 	char *help;
