@@ -1,6 +1,8 @@
 /*
-  getcwd -- get working directory pathname
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  $NiH: getcwd.c,v 1.9 2002/09/16 12:42:35 dillo Exp $
+
+  getcwd.c -- get working directory pathname
+  Copyright (C) 1996-2002 Dieter Baron
 
   The author can be contacted at <dillo@giga.or.at>
 
