@@ -1,6 +1,8 @@
 /*
-  inet_aton -- replacement function (using inet_addr)
-  Copyright (C) 2000 Dieter Baron
+  $NiH: inet_aton.c,v 1.4 2002/09/16 12:42:35 dillo Exp $
+
+  inet_aton.c -- replacement function (using inet_addr)
+  Copyright (C) 2000-2002 Dieter Baron
 
   The author can be contacted at <dillo@giga.or.at>
 
