@@ -2,8 +2,10 @@
 #define HAD_KEYS_H
 
 /*
+  $NiH: keys.h,v 1.9 2002/09/16 12:42:36 dillo Exp $
+
   keys.h -- parse and print key specifications
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -35,7 +37,7 @@ struct fnkey {
     char *cap, *seq;
 };
 
-extern struct keyname keyname[];
+extern struct keyname key_name[];
 extern struct fnkey fnkey[];
 extern int max_fnkey;
 
