@@ -1,6 +1,8 @@
 /*
-  list -- display lists
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  $NiH: list.c,v 1.20 2002/09/17 11:59:46 dillo Exp $
+
+  list.c -- display lists
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -22,6 +24,9 @@
 
 
 
+#include <stdlib.h>
+#include <string.h>
+
 #include "display.h"
 #include "list.h"
 #include "tty.h"
