@@ -1,6 +1,8 @@
 /*
-  loop -- main loop
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  $NiH: loop.c,v 1.16 2002/09/16 12:42:36 dillo Exp $
+
+  loop.c -- main loop
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -60,7 +62,7 @@ loop()
 		return;
 
 	    if (f->type != FN_PRE)
-		disp_status("");
+		disp_status(DISP_STATUS, "");
 
 	    if (f->type != FN_EXIT)
 		f->fn(binding->args);
@@ -69,7 +71,7 @@ loop()
 		void_prefix();
 	}
 	else {
-	    disp_status("");
+	    disp_status(DISP_STATUS, "");
 	    void_prefix();
 	}
 
