@@ -1,6 +1,8 @@
 /*
-  main -- main function
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  $NiH: main.c,v 1.47 2002/09/16 12:42:37 dillo Exp $
+
+  main.c -- main function
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -32,7 +34,7 @@
 #include <getopt.h>
 #include <unistd.h>
 
-
+#include "config.h"
 #include "directory.h"
 #include "display.h"
 #include "loop.h"
@@ -46,6 +48,7 @@
 #include "status.h"
 #include "tty.h"
 #include "util.h"
+#include "url.h"
 
 /* in readrc.c */
 int readrc(char **userp, char **passp, char **hostp, char **portp,
@@ -55,7 +58,11 @@ int readrc(char **userp, char **passp, char **hostp, char **portp,
 
 char *usage[] = {
 	"{-h|-V}",
-	"[-p port] [-u user] {host|alias} [directory]",
+	"[-p port] [-u user] "
+#ifdef USE_SFTP
+	 "[-s] "
+#endif
+	 "{host|alias} [directory]",
 	"url",
 	NULL };
 
@@ -65,34 +72,43 @@ char help[] = "\
   -h, --help        display this help message\n\
   -V, --version     display version number\n\
   -p, --port PORT   specify port\n\
-  -u, --user USER   specify user\n\
-\n\
+  -u, --user USER   specify user\n"
+#ifdef USE_SFTP
+"  -s, --sftp        use sftp\n"
+#endif
+"\n\
 Report bugs to <dillo@giga.or.at>.\n";
 
 char version_tail[] = "\
-Copyright (C) 2000 Dieter Baron\n\
+Copyright (C) 2002 Dieter Baron\n\
 cftp comes with ABSOLUTELY NO WARRANTY, to the extent permitted by law.\n\
 You may redistribute copies of\n\
 cftp under the terms of the GNU General Public License.\n\
 For more information about these matters, see the files named COPYING.\n";
 
+#ifdef USE_SFTP
+#define OPTIONS	"hVp:u:s"
+#else
 #define OPTIONS	"hVp:u:"
+#endif
 
 struct option options[] = {
     { "help",      0, 0, 'h' },
     { "version",   0, 0, 'V' },
     { "port",      1, 0, 'p' },
     { "user",      1, 0, 'u' },
+#ifdef USE_SFTP
+    { "sftp",      1, 0, 's' },
+#endif
     { NULL,        0, 0, 0   }
 };
 
 char *prg;
-extern char version[];
+char version[] = PACKAGE " " VERSION;
 
 
 
 void print_usage(FILE *f);
-char *get_anon_passwd(void);
 void read_netrc(char *host, char **user, char **pass, char **wdir);
 void sig_end(int i);
 void sig_escape(int i);
@@ -108,9 +124,8 @@ main(int argc, char **argv)
 
     directory *dir;
     char *host, *user = NULL, *port = NULL, *pass = NULL, *wdir = NULL;
-    char *poss_fn;
-    int keep_pass = 1;
-    int c, sel;
+    char *poss_fn, *poss_dir;
+    int c;
     char *b, *b2;
     int check_alias;
 
@@ -131,6 +146,8 @@ main(int argc, char **argv)
 	exit(1);
     status_init(); /* can't fail */
 
+    ftp_proto = 0;
+
     opterr = 0;
     while ((c=getopt_long(argc, argv, OPTIONS, options, 0)) != EOF) {
 	switch (c) {
@@ -140,6 +157,11 @@ main(int argc, char **argv)
 	case 'u':
 	    user = strdup(optarg);
 	    break;
+#ifdef USE_SFTP
+	case 's':
+	    ftp_proto = 1;
+	    break;
+#endif
 	case 'V':
 	    printf("%s\n", version);
 	    fputs(version_tail, stdout);
@@ -159,13 +181,14 @@ main(int argc, char **argv)
 	print_usage(stderr);
 	exit(1);
     }
-    
-    if (strncmp(argv[optind], "ftp://", 6) == 0) {
+
+    if (is_url(argv[optind])) {
 	if (argc > optind+1) {
 	    print_usage(stderr);
 	    exit(1);
 	}
-	if (parse_url(argv[optind], &user, &pass, &host, &port, &wdir) < 0)
+	if (parse_url(argv[optind], &ftp_proto, &user, &pass,
+		      &host, &port, &wdir) < 0)
 	    exit(1);
 	
 	check_alias = 0;
@@ -186,41 +209,26 @@ main(int argc, char **argv)
 
     /* XXX */ readrc(&user, &pass, &host, &port, &wdir, check_alias);
 
-    if (user == NULL) {
-	read_netrc(host, &user, &pass, &wdir);
-    }
-    
-    if (user == NULL ||
-	(strcmp(user, "ftp") == 0 || strcmp(user, "anonymous") == 0)) {
-	if (user == NULL)
-	    user = "ftp";
-	if (pass == NULL)
-	    pass = get_anon_passwd();
-    }
-    else {
-	if (pass == NULL) {
-	    b = (char *)malloc(strlen(user)+strlen(host)+16);
-	    sprintf(b, "Password (%s@%s): ", user, host);
-	    pass = (char *)getpass(b);
-	    keep_pass = 0;
-	    free(b);
+    if (ftp_proto == 0) {
+	if (user == NULL) {
+	    read_netrc(host, &user, &pass, &wdir);
+	}
+	
+	if (user == NULL ||
+	    (strcmp(user, "ftp") == 0 || strcmp(user, "anonymous") == 0)) {
+	    if (user == NULL)
+		user = "ftp";
+	    if (pass == NULL)
+		pass = get_anon_passwd();
 	}
     }
-    if (port == NULL)
-	port = "ftp";
     
     if (tty_init() < 0)
 	exit(1);
 
     ftp_init();
 
-    ftp_host = host;
-    ftp_prt = port;
-    ftp_user = user;
-    if (keep_pass)
-	ftp_pass = pass;
-
-    if (ftp_open(ftp_host, ftp_prt) == -1)
+    if (ftp_open(host, port, user, pass) == -1)
 	exit(1);
 
     if (init_disp() < 0)
@@ -232,7 +240,7 @@ main(int argc, char **argv)
 
     binding_state = bs_remote;
 
-    if (ftp_login(host, user, pass) == -1) {
+    if (ftp_login(user, pass) == -1) {
 	exit_disp();
 	exit(1);
     }
@@ -260,19 +268,13 @@ main(int argc, char **argv)
     if ((dir=ftp_cd(wdir, 0)) == NULL) {
 	if ((poss_fn=strrchr(wdir, '/')) != NULL) {
 	    *(poss_fn++)=0;
-	    if ((dir=ftp_cd(strlen(wdir)?wdir:"/", 0)) != NULL) {
-		curdir = dir;
-		list = (struct list *)curdir;
-		if ((sel=dir_find(curdir, poss_fn)) >= 0) {
-		    aux_scroll(sel-(win_lines/2), sel, 0);
-		    list_do(1);
-		    if (curdir->line[sel].type == 'f'
-			|| curdir->line[sel].type == 'l')
-			aux_download(curdir->line[sel].name,
-				     curdir->line[sel].size, 0);
-		}
-		else
-		    disp_status("can't find file `%s'", poss_fn);
+	    poss_dir = strlen(wdir) ? wdir : "/";
+	    if (ftp_cwd(poss_dir) == 0) {
+		ftp_lcwd = strdup(poss_dir);
+		free(status.remote.path);
+		status.remote.path = strdup(ftp_lcwd);
+		status_do(bs_remote);
+		aux_download(poss_fn, -1, 0);
 	    }
 	}
 	if (dir == NULL) {
@@ -312,36 +314,6 @@ print_usage(FILE *f)
 
 
 
-char *
-get_anon_passwd(void)
-{
-    char pass[8192], host[1024], domain[1024];
-    struct passwd *pwd;
-
-    pwd = getpwuid(getuid());
-
-    if (pwd)
-	sprintf(pass, "%s@", pwd->pw_name);
-    else
-	strcpy(pass, "unknown@");
-
-    gethostname(host, 1023);
-    getdomainname(domain, 1023);
-
-    if (strcmp(domain, "(none)") != 0 && domain[0] != '\0') {
-	if (domain[0] != '.')
-	    sprintf(pass+strlen(pass), "%s.%s", host, domain);
-	else
-	    sprintf(pass+strlen(pass), "%s%s", host, domain);
-    }
-    else if (strchr(host, '.'))
-	strcat(pass, host);
-    
-    return strdup(pass);
-}
-
-
-
 void
 read_netrc(char *host, char **user, char **pass, char **wdir)
 {
