@@ -1,6 +1,8 @@
 /*
-  mkbind -- make binding table
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  $NiH: mkbind.c,v 1.23 2002/09/17 11:46:23 dillo Exp $
+
+  mkbind.c -- make binding table
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -22,16 +24,20 @@
 
 
 
+#include <sys/types.h>
+#include <errno.h>
+#include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <errno.h>
+#include <unistd.h>
 
+#include "bindings.h"
+#include "display.h"
+#include "fntable.h"
+#include "functions.h"
 #include "keys.h"
 #include "rc.h"
-#include "bindings.h"
-#include "functions.h"
-#include "fntable.h"
 
 #define FNAME	"bindings"
 #define NAME	"binding"
@@ -61,7 +67,6 @@ int nstates = sizeof(states)/sizeof(states[0]);
 function functions[MAX_FN];
 int num;
 
-enum state binding_state;
 struct binding binding[MAX_FN];
 
 char *binding_argpool[1];
@@ -82,6 +87,7 @@ main(int argc, char **argv)
 {
     FILE *fin, *fout;
     char line[4069], *p, *tok, **args;
+    char tmp[128];
     struct binding *b;
     int i, j, off, argoff, len;
     int maxkey;
@@ -138,11 +144,13 @@ main(int argc, char **argv)
     rc_filename = NULL;
 
 
-    /* writing ``bindings.c'' */
+    /* writing ``bindings.c'' in temp file */
 
-    if ((fout=fopen(FNAME ".c", "w")) == NULL) {
+    sprintf(tmp, FNAME ".c.%d", getpid());
+
+    if ((fout=fopen(tmp, "w")) == NULL) {
 	fprintf(stderr, "%s: can't open output file `%s': %s.\n",
-		prg, FNAME ".c", strerror(errno));
+		prg, tmp, strerror(errno));
 	exit(1);
     }
 
@@ -246,11 +254,20 @@ main(int argc, char **argv)
     if (ferror(fout)) {
 	fprintf(stderr, "%s: write error on `%s': %s.\n",
 		prg, FNAME ".c", strerror(errno));
-	exit(1);
 	fclose(fout);
+	exit(1);
     }
 
     fclose(fout);
+
+    /* rename tmp file to ``bindings.c'' */
+
+    if (rename(tmp, FNAME ".c") < 0) {
+	fprintf(stderr, "%s: cannot rename `%s' to `%s': %s\n",
+		prg, tmp, FNAME ".c", strerror(errno));
+	exit(1);
+    }
+
     exit(0);
 }
 
@@ -268,7 +285,7 @@ initnames(void)
 	exit(1);
     }
 
-    rc_lineno = 0;
+    rc_lineno = 1;
     while (fgets(line, 8192, f) && line[strlen(line)-2] != '{')
 	rc_lineno++;
 
@@ -396,8 +413,16 @@ vpath_open(char *name)
 /* dummy functions needed by fn_bind */
 
 void
-disp_status(char *fmt, ...)
+disp_status(int flags, char *fmt, ...)
 {
+    va_list argp;
+    
+    if (flags & DISP_STDERR) {
+	va_start(argp, fmt);
+	vfprintf(stderr, fmt, argp);
+	va_end(argp);
+    }
+    
     return;
 }
 
