@@ -1,6 +1,10 @@
 /*
    This file is automatically created from ``options.op''; don't make
    changes to this file, change ``options.op'' instead.
+
+  Created from:
+    $NiH: mkoptab-c.m4,v 1.10 2002/09/16 12:42:39 dillo Exp $
+    $NiH: options.op,v 1.23 2002/09/16 12:42:40 dillo Exp $
 */
 
 #include <stddef.h>
@@ -11,6 +15,8 @@ extern void opts_mode();
 
 int opt_pasv = 0;
 
+int opt_stall = 900;
+
 int opt_tagchar = '>';
 
 int ftp_hist_size = 200;
@@ -28,12 +34,16 @@ extern void opt_set_sort();
 
 int opt_wrap = 1;
 
+int opt_user_anon_passwd = 0;
+
 
 struct uoption option[] = {
   { "mode" , "mo", "transfer mode for downloads",
     OPT_CHR, (void *)&opt_mode, opts_mode, NULL },
   { "passive" , "pv", "use passive mode to establish data connections",
     OPT_BOOL, (void *)&opt_pasv, NULL, NULL },
+  { "stall-timeout" , "st", "timeout for stalled connections (in seconds)",
+    OPT_INT, (void *)&opt_stall, NULL, NULL },
   { "tag-char" , "tc", "char to mark tagged files with",
     OPT_CHR, (void *)&opt_tagchar, NULL, NULL },
   { "hist-size" , "hs", "size of ftp command/response history",
@@ -48,5 +58,7 @@ struct uoption option[] = {
     OPT_ENUM, (void *)&opt_sort, opt_set_sort, openum_sort },
   { "wrap" , "wa", "wrap around bottom/top when scrolling",
     OPT_BOOL, (void *)&opt_wrap, NULL, NULL },
+  { "user-anon-passwd" , "ua", "whether to send user name as anonymous password",
+    OPT_BOOL, (void *)&opt_user_anon_passwd, NULL, NULL },
   NULL, NULL, NULL, 0, NULL, NULL
 };
