@@ -3,6 +3,10 @@
 /*
    This file is automatically created from ``options.op''; don't make
    changes to this file, change ``options.op'' instead.
+
+  Created from:
+    $NiH: mkoptab-h.m4,v 1.9 2002/09/16 12:42:39 dillo Exp $
+    $NiH: options.op,v 1.23 2002/09/16 12:42:40 dillo Exp $
 */
 
 #define OPT_INT 0
@@ -26,9 +30,9 @@ struct uoption {
 };
 
 extern struct uoption option[];
-
 extern int opt_mode;
 extern int opt_pasv;
+extern int opt_stall;
 extern int opt_tagchar;
 extern int ftp_hist_size;
 extern int opt_beep;
@@ -36,5 +40,6 @@ extern int opt_scrlimit;
 extern char *opt_pager;
 extern int opt_sort;
 extern int opt_wrap;
+extern int opt_user_anon_passwd;
 
 #endif /* options.h */
