@@ -1,6 +1,8 @@
 /*
-  rc -- auxiliary functions for parsing .cftprc file
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  $NiH: rc.c,v 1.12 2002/09/16 12:42:40 dillo Exp $
+
+  rc.c -- auxiliary functions for parsing .cftprc file
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
