@@ -1,6 +1,8 @@
 /*
-  readdir -- read directory listing
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  $NiH: readdir.c,v 1.24 2002/09/16 12:42:41 dillo Exp $
+
+  readdir.c -- read directory listing
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -29,6 +31,7 @@
 
 #include "directory.h"
 #include "ftp.h"
+#include "display.h"
 #include "options.h"
 
 static int parse_unix(direntry *de, char *line);
@@ -51,27 +54,21 @@ directory *
 read_dir(FILE *f)
 {
     directory *dir;
-    direntry *list;
-    int i, n, sz, pf, ret;
+    direntry entry;
+    int n, pf, ret;
     char *line;
+    time_t oldt, newt;
 
-    dir = malloc(sizeof(directory));
-    list = NULL;
-    sz = n = 0;
+    dir = dir_new();
+    n = 0;
 
     pf = 0;
+    oldt = 0;
 
     init_parse_time();
 
     while ((line=ftp_gets(f)) != NULL) {
-	if (n >= sz) {
-	    sz += DIR_STEP;
-	    if (list == NULL)
-		list = malloc(sizeof(direntry)*sz);
-	    else
-		list = realloc(list, sizeof(direntry)*sz);
-	}
-	while ((ret=pfunc[pf](list+n, line)) == -1) {
+	while ((ret=pfunc[pf](&entry, line)) == -1) {
 	    pf++;
 	    if (pf >= npfunc) {
 		pf = 0;
@@ -80,12 +77,18 @@ read_dir(FILE *f)
 	    }
 	}
 	if (ret == 0) {
-	    list[n].pos = n;
+	    dir_add(dir, &entry);
 	    n++;
 	}
 	free(line);
+
+	if ((newt=time(NULL)) != oldt) {
+	    disp_status(DISP_STATUS, "listed %d", n);
+	    oldt = newt;
+	}
     }
 
+#if 0
     if (n == 0) {
 	dir->line = (direntry *)malloc(sizeof(direntry));
 	dir->line->line = strdup("");
@@ -94,15 +97,7 @@ read_dir(FILE *f)
 	dir->line->link = NULL;
 	n = 1;
     }
-    else {
-	dir->line = malloc(sizeof(direntry)*n);
-	for (i=0; i<n; i++)
-	    dir->line[i] = list[i];
-    }
-    dir->sorted = 0;
-    dir->len = n;
-    dir->size = sizeof(struct direntry);
-    dir->top = dir->cur = 0;
+#endif
 
     return dir;
 }
