@@ -1,6 +1,8 @@
 /*
-  readrc -- read .cftprc file
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  $NiH: readrc.c,v 1.17 2002/09/16 12:42:41 dillo Exp $
+
+  readrc.c -- read .cftprc file
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -29,8 +31,12 @@
 
 #include "util.h"
 #include "bindings.h"
+#include "ftp.h"
 #include "functions.h"
 #include "rc.h"
+#include "url.h"
+
+
 
 int
 readrc(char **userp, char **passp, char **hostp, char **portp, char **wdirp, 
@@ -73,9 +79,9 @@ readrc(char **userp, char **passp, char **hostp, char **portp, char **wdirp,
 		continue;
 
 	    if ((tok=rc_token(&p)) != NULL) {
-		if (strncmp(tok, "ftp://", 6) == 0) {
-		    parse_url(tok, &user, &pass, &host, &port, &wdir);
-		}
+		if (is_url(tok))
+		    parse_url(tok, &ftp_proto, &user, &pass,
+			      &host, &port, &wdir);
 		else {
 		    user = pass = port = NULL;
 		    host = strdup(tok);
