@@ -1,6 +1,8 @@
 /*
-  sockets -- auxiliary socket functions
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  $NiH: sockets.c,v 1.25 2002/09/16 12:42:43 dillo Exp $
+
+  sockets.c -- auxiliary socket functions
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -37,8 +39,6 @@
 #include "display.h"
 #include "sockets.h"
 
-extern char *prg;
-
 
 
 int
@@ -53,12 +53,8 @@ sopen(char *host, char *service, int family)
     hints.ai_socktype = SOCK_STREAM;
     
     if ((err=getaddrinfo(host, service, &hints, &res0)) != 0) {
-	if (disp_active)
-	    disp_status("cannot get host/service %s/%s: %s\n",
-			host, service, gai_strerror(err));
-	else
-	    fprintf(stderr, "%s: can't get host/service %s/%s: %s\n",
-		    prg, host, service, gai_strerror(err));
+	disp_status(DISP_ERROR, "cannot get host/service %s/%s: %s\n",
+		    host, service, gai_strerror(err));
 	return -1;
     }
 
@@ -80,14 +76,9 @@ sopen(char *host, char *service, int family)
 	/* okay we got one */
 	break;
     }
-    if (s < 0) {
-	if (disp_active)
-	    disp_status("cannot %s: %s\n",
-			cause, strerror(errno));
-	else
-	    fprintf(stderr, "%s: cannot %s: %s\n",
-		    prg, cause, strerror(errno));
-    }
+    if (s < 0)
+	disp_status(DISP_ERROR, "cannot %s: %s", cause, strerror(errno));
+
     freeaddrinfo(res0);
 
     return s;
@@ -108,12 +99,8 @@ spassive(int family, struct sockaddr *addr, int *lenp)
     hints.ai_flags = AI_PASSIVE;
 
     if (getaddrinfo(NULL, "0", &hints, &res0) != 0) {
-	if (disp_active)
-	    disp_status("cannot get address info: %s\n",
-			gai_strerror(errno));
-	else
-	    fprintf(stderr, "%s: cannot get address info: %s\n",
-		    prg, gai_strerror(errno));
+	disp_status(DISP_ERROR, "cannot get address info: %s\n",
+		    gai_strerror(errno));
 	return -1;
     }
 	
@@ -148,11 +135,7 @@ spassive(int family, struct sockaddr *addr, int *lenp)
     
     freeaddrinfo(res0);
 
-    if (disp_active)
-	disp_status("cannot %s: %s", cause, strerror(errno));
-    else
-	fprintf(stderr, "%s: cannot %s: %s",
-		prg, cause, strerror(errno));
+    disp_status(DISP_ERROR, "cannot %s: %s", cause, strerror(errno));
     return -1;
 }
 
@@ -161,7 +144,7 @@ spassive(int family, struct sockaddr *addr, int *lenp)
 const char *
 sockaddr_ntop(struct sockaddr *sa)
 {
-#ifdef HAVE_GETADDRINFO
+#ifdef HAVE_GETNAMEINFO
 
     static char addrbuf[NI_MAXHOST];
     int len;
@@ -181,7 +164,7 @@ sockaddr_ntop(struct sockaddr *sa)
     else
         return NULL;
 
-#else /* !HAVE_GETADDRINFO */
+#else /* !HAVE_GETNAMEINFO */
 
     struct sockaddr_in *si;
     if (sa->sa_family != AF_INET)
