@@ -1,6 +1,8 @@
 /*
-  status.h -- status line
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  $NiH: status.c,v 1.19 2002/09/16 12:42:43 dillo Exp $
+
+  status.c -- status line
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -155,14 +157,6 @@ status_do(enum state when)
 
 
 
-void
-opt_set_status(int optval, int *optvar)
-{
-    /* XXX */
-}
-
-
-
 extern enum state leave_tag;
 
 void
@@ -177,11 +171,11 @@ enter_state(enum state state)
 	break;
 
     case bs_local:
-	disp_status("state <local> not implemented yet");
+	disp_status(DISP_STATUS, "state <local> not implemented yet");
 
     case bs_tag:
 	if (!tag_anytags()) {
-	    disp_status("no tags");
+	    disp_status(DISP_STATUS, "no tags");
 	    break;
 	}
 	if (binding_state != bs_tag)
@@ -193,10 +187,10 @@ enter_state(enum state state)
 	break;
 
     case bs_none:
-	disp_status("can't enter state <global>");
+	disp_status(DISP_STATUS, "can't enter state <global>");
 	break;
 
     default:
-	disp_status("no such state: %s", state);
+	disp_status(DISP_STATUS, "no such state: %s", state);
     }
 }
