@@ -2,8 +2,10 @@
 #define HAD_STATUS_H
 
 /*
+  $NiH: status.h,v 1.10 2002/09/16 12:42:43 dillo Exp $
+
   status.h -- status line
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -23,6 +25,8 @@
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
+#include "bindings.h"
+
 
 
 struct status {
