@@ -1,6 +1,8 @@
 /*
-  strdup -- save a copy of a string
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  $NiH: strdup.c,v 1.10 2002/09/16 12:42:44 dillo Exp $
+
+  strdup.c -- save a copy of a string
+  Copyright (C) 1996-2002 Dieter Baron
 
   The author can be contacted at <dillo@giga.or.at>
 
@@ -21,6 +23,7 @@
 
 
 
+#include <stdlib.h>
 #include <string.h>
 
 char *
