@@ -1,6 +1,8 @@
 /*
-  strerror -- get error message string
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  $NiH: strerror.c,v 1.9 2002/09/16 12:42:44 dillo Exp $
+
+  strerror.c -- get error message string
+  Copyright (C) 1996-2002 Dieter Baron
 
   The author can be contacted at <dillo@giga.or.at>
 
