@@ -1,6 +1,8 @@
 /*
-  tag -- tagging
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  $NiH: tag.c,v 1.36 2002/09/16 12:42:44 dillo Exp $
+
+  tag.c -- tagging
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -26,6 +28,16 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "config.h"
+
+#ifdef HAVE_BASENAME
+# ifdef HAVE_LIBGEN_H
+#  include <libgen.h>
+# endif
+#else
+char *basename(char *);
+#endif
+
 #include "tag.h"
 #include "directory.h"
 #include "options.h"
@@ -222,7 +234,7 @@ _tag_insert(int n, struct tagentry *t, char *file, long size, char type)
 
     u->line = line;
     u->name = file;
-    u->file = (char *)basename(file);
+    u->file = basename(file);
     u->dirl = tags.line[n].file - file;
     if (u->dirl > 1)
 	--u->dirl;
