@@ -2,8 +2,10 @@
 #define HAD_TAG_H
 
 /*
+  $NiH: tag.h,v 1.13 2002/09/16 12:42:45 dillo Exp $
+
   tag.h -- tagging
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -45,7 +47,7 @@ struct taglist {
 
 enum tagopt { TAG_ON, TAG_OFF, TAG_TOGGLE };
 
-struct taglist tags;
+extern struct taglist tags;
 
 
 
