@@ -1,6 +1,8 @@
 /*
-  tty -- lowlevel tty handling
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  $NiH: tty.c,v 1.24 2002/09/16 12:42:45 dillo Exp $
+
+  tty.c -- lowlevel tty handling
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -31,6 +33,23 @@
 #include <sys/ioctl.h>
 
 #include "config.h"
+#ifdef HAVE_TERMCAP_H
+#include <termcap.h>
+#endif
+#ifndef HAVE_DECL_OSPEED
+char PC, *BC, *UP;
+short ospeed;
+#endif
+
+#ifndef HAVE_FPUTCHAR
+int fputchar(int c);
+#endif
+
+#if !defined(HAVE_TPARAM) && !defined(HAVE_TPARM)
+char *tparam (char *string, char *outstring, int len,
+	      int arg0, int arg1, int arg2, int arg3);
+#endif
+
 #include "keys.h"
 #include "tty.h"
 
@@ -62,8 +81,6 @@ char termcap_entry[4196];
 char termcap_area[4196], *termcap_str;
 struct termios tty_tio, tty_tio_ext;
 speed_t tty_baud;
-char PC, *BC, *UP;
-short ospeed;
 
 #define KEY_PREF	1
 #define KEY_SLPREF	2
@@ -243,6 +260,14 @@ tty_put(char *name, int lines)
 
 
 
+void
+tty_put0(char *cap, int lines)
+{
+    tputs(cap, lines, fputchar);
+}
+
+
+
 void
 tty_goto(int x, int y)
 {
@@ -324,7 +349,7 @@ tty_readkey(void)
     len = 0;
     
     if (keyflag[(int)s[0]] == KEY_PREF) {
-	while ((c=tty_ispref(s, l)) == EOF) {
+	while ((c=tty_ispref((char *)s, l)) == EOF) {
  	    if (!vmin) {
 		tty_vmin(0, 5);
 		vmin = 1;
@@ -501,7 +526,9 @@ _tty_capinit()
 void
 tty_putp(char *cap, int lines, int arg0, int arg1, int arg2, int arg3)
 {
+#ifndef HAVE_TPARM
     static char buf[40];
+#endif
     char *s;
 
 #ifndef HAVE_TPARM
@@ -524,7 +551,7 @@ void
 tty_lowleft(void)
 {
     if (*TTY_CAP(ll))
-	tty_put0(ll, 1);
+	tty_put0(TTY_CAP(ll), 1);
     else
 	tty_goto(0, tty_lines-1);
 }
