@@ -2,8 +2,10 @@
 #define HAD_TTY_H
 
 /*
+  $NiH: tty.h,v 1.17 2002/09/16 12:42:45 dillo Exp $
+
   tty.h -- lowlevel tty handling
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -32,20 +34,17 @@ enum {
     _TTY_al, _TTY_dl, _TTY_AL, _TTY_DL   
 };
 extern char *_tty_caps[];
-extern int fputchar();
 
 #define TTY_CAP(cap)	(_tty_caps[(_TTY_##cap)])
-#define tty_put0(cap, lines)  \
-		(tputs(TTY_CAP(cap), (lines), (void (*)())fputchar))
 
-#define tty_clear()	(tty_put0(cl, tty_lines))
-#define tty_home()	(tty_put0(ho, 1))
-#define tty_clreos(l)	(tty_put0(cd, (l)))
-#define tty_clreol()	(tty_put0(ce, 1))
-#define tty_standout()	(tty_put0(so, 1))
-#define tty_standend()	(tty_put0(se, 1))
-#define tty_hidecrsr()	(tty_put0(vi, 1))
-#define tty_showcrsr()	(tty_put0(ve, 1))
+#define tty_clear()	(tty_put0(TTY_CAP(cl), tty_lines))
+#define tty_home()	(tty_put0(TTY_CAP(ho), 1))
+#define tty_clreos(l)	(tty_put0(TTY_CAP(cd), (l)))
+#define tty_clreol()	(tty_put0(TTY_CAP(ce), 1))
+#define tty_standout()	(tty_put0(TTY_CAP(so), 1))
+#define tty_standend()	(tty_put0(TTY_CAP(se), 1))
+#define tty_hidecrsr()	(tty_put0(TTY_CAP(vi), 1))
+#define tty_showcrsr()	(tty_put0(TTY_CAP(ve), 1))
 
 void tty_lowleft(void);
 
@@ -62,7 +61,7 @@ extern volatile int tty_cols, tty_lines;
 
 enum tty_am { TTY_AMNONE, TTY_AMXN, TTY_AM };
 
-enum tty_am tty_am;
+extern enum tty_am tty_am;
 
 extern int tty_metap, tty_noLP,
 	   tty_verase, tty_vwerase, tty_vkill;
@@ -75,6 +74,7 @@ int tty_init(void);
 int tty_setup(void);
 int tty_restore(void);
 void tty_put(char *name, int lines);
+void tty_put0(char *cap, int lines);
 void tty_goto(int x, int y);
 int tty_noecho(void);
 int tty_echo(void);
