@@ -1,6 +1,8 @@
 /*
-  util -- auxiliary functions
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  $NiH: util.c,v 1.22 2002/09/17 14:58:19 dillo Exp $
+
+  util.c -- auxiliary functions
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -22,6 +24,7 @@
 
 
 
+#include <fcntl.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -29,6 +32,12 @@
 #include <sys/types.h>
 #include <unistd.h>
 
+#include "directory.h"
+#include "ftp.h"
+#include "options.h"
+#include "util.h"
+#include "url.h"
+
 
 
 char *
@@ -80,7 +89,7 @@ canonical(char *path, char *current)
 
 
 char *
-dirname(char *name)
+xdirname(const char *name)
 {
     char *dir;
     char *p = strrchr(name, '/');
@@ -141,108 +150,6 @@ local_exp(char *path)
 
 
 
-static char *deurl(char *u);
-
-int
-parse_url(char *url, char **user, char **pass,
-	  char **host, char **port, char **dir)
-{
-    char *p, *q, *h, *r;
-
-    if (strncmp(url, "ftp://", 6) != 0)
-	return -1;
-    url+=6;
-
-    if ((p=strchr(url, '/')) != NULL)
-	*(p++) = '\0';
-    else
-	p = url+strlen(url);
-    
-    if ((q=strrchr(url, '@')) != NULL) {
-	*q = '\0';
-	if ((r=strchr(url, ':')) != NULL) {
-	    *(r++) = '\0';
-	    *pass = deurl(r);
-	}
-	*user = deurl(url);
-	url = q+1;
-    }
-
-    if (url[0] == '[' && (h=strchr(url, ']')) != NULL
-	&& (h[1] == ':' || h[1] == '\0')) {
-	url++;
-	*(h++) = '\0';
-    }
-    else
-	h = url;
-
-    if ((q=strchr(h, ':')) != NULL) {
-	*q = '\0';
-	if (*(q+1) != '\0')
-	    *port = deurl(q+1);
-    }
-	
-    *host = deurl(url);
-
-    if (p && *p != '\0') {
-	if ((*dir=(char *)malloc(strlen(p)+3)) == NULL) {
-	    /* fprintf(stderr, "%s: malloc failure\n", prg); */
-	    fprintf(stderr, "malloc failure\n");
-	    return -1;
-	}
-	*dir = deurl(p);
-    }
-
-    return 0;
-}
-
-
-
-static int
-hexdigit(int c)
-{
-    if (c >= '0' && c <= '9')
-	return c-'0';
-    if (c >= 'a' && c <= 'f')
-	return c-'a';
-    if (c >= 'A' && c <= 'F')
-	return c-'F';
-    
-    return 0;
-}
-
-
-
-static char *
-deurl(char *s)
-{
-    char *t, *p;
-    int c;
-
-    if ((t=(char *)malloc(strlen(s)+1)) != NULL) {
-	for (p=t; *s; s++) {
-	    if (*s == '%') {
-		if (s[1] == '\0' || s[2] == '\0')
-		    *(p++) = '%';
-		else {
-		    c = hexdigit(*(++s))*16;
-		    c += hexdigit(*(++s));
-
-		if (c != 0)
-		    *(p++) = c;
-		}
-	    }
-	    else
-		    *(p++) = *s;
-	}
-	*p = '\0';
-    }
-
-    return t;
-}
-
-
-
 char *
 args_to_string(char **args)
 {
@@ -266,3 +173,127 @@ args_to_string(char **args)
 
     return s;
 }
+
+
+
+char *
+get_anon_passwd(void)
+{
+    char pass[8192], host[1024], domain[1024];
+    struct passwd *pwd;
+
+    if (!opt_user_anon_passwd)
+	return strdup("anonymous@");
+    else {
+	pwd = getpwuid(getuid());
+
+	if (pwd)
+	    sprintf(pass, "%s@", pwd->pw_name);
+	else
+	    strcpy(pass, "unknown@");
+	
+	gethostname(host, 1023);
+#ifdef HAVE_GETDOMAINNAME
+	getdomainname(domain, 1023);
+#else
+	domain[0] = '\0';
+#endif
+	
+	if (strcmp(domain, "(none)") != 0 && domain[0] != '\0') {
+	    if (domain[0] != '.')
+		sprintf(pass+strlen(pass), "%s.%s", host, domain);
+	    else
+		sprintf(pass+strlen(pass), "%s%s", host, domain);
+	}
+	else if (strchr(host, '.'))
+	    strcat(pass, host);
+	
+	return strdup(pass);
+    }
+}
+
+
+
+char *
+mkhoststr(int passp, int urlp)
+{
+    int len, i, n;
+    char *str[7], *hs, *t;
+    int encode[7], slen[7];
+
+    n = 0;
+    len = 0;
+    if ((ftp_user() && !ftp_anon()) || (passp && ftp_pass())) {
+	str[n] = ftp_user();
+	encode[n++] = urlp;
+	if (passp && ftp_pass()) {
+	    str[n] = ":";
+	    encode[n++] = 0;
+	    str[n] = ftp_pass();
+	    encode[n++] = urlp;
+	}
+	str[n] = "@";
+	encode[n++] = 0;
+    }
+    str[n] = ftp_host();
+    encode[n++] = 1;
+    if (ftp_prt()) {
+	str[n] = ":";
+	encode[n++] = 0;
+	str[n] = ftp_prt();
+	encode[n++] = urlp;
+    }
+
+    len = 0;
+    for (i=0; i<n; i++) {
+	if (encode[i])
+	    slen[i] = url_enclen(str[i], URL_UCHAR);
+	else
+	    slen[i] = strlen(str[i]);
+	len += slen[i];
+    }
+    len++;
+
+    t = hs = (char *)malloc(len);
+
+    len = 0;
+    for (i=0; i<n; i++) {
+	if (encode[i])
+	    url_encode(t, str[i], URL_UCHAR);
+	else
+	    strcpy(t, str[i]);
+	t += slen[i];
+    }
+    *t = '\0';
+
+    return hs;
+}
+
+
+
+/*
+  Set FD to blocking if BLOCKING, non-blocking otherwise.  Return 0 if
+  FD was in non-blocking mode, 1 if it was in blocking mode, -1 in
+  case of error.
+*/
+
+int
+set_file_blocking(int fd, int blocking)
+{
+    int ret, flags;
+    
+    if ((flags=fcntl(fd, F_GETFL, 0)) == -1)
+	return -1;
+
+    ret = flags & O_NONBLOCK ? 0 : 1;
+
+    if (blocking)
+	flags &= ~O_NONBLOCK;
+    else
+	flags |= O_NONBLOCK;
+
+    if (fcntl(fd, F_SETFL, flags) == -1)
+	return -1;
+
+    return ret;
+}
