@@ -2,8 +2,10 @@
 #define HAD_UTIL_H
 
 /*
+  $NiH: util.h,v 1.17 2002/09/17 14:58:19 dillo Exp $
+
   util.h -- auxiliary functions
-  Copyright (C) 1996, 1997, 1998, 1999, 2000 Dieter Baron
+  Copyright (C) 1996-2002 Dieter Baron
 
   This file is part of cftp, a fullscreen ftp client
   The author can be contacted at <dillo@giga.or.at>
@@ -25,11 +27,12 @@
 
 
 
-char *canonical(char *path, char *current);
-char *dirname(char *name);
-char *local_exp(char *path);
-int parse_url(char *url, char **user, char **pass,
-	      char **host, char **port, char **dir);
 char *args_to_string(char **args);
+char *canonical(char *path, char *current);
+char *xdirname(const char *name);
+char *get_anon_passwd(void);
+char *local_exp(char *path);
+char *mkhoststr(int passp, int urlp);
+int set_file_blocking(int fd, int blocking);
 
 #endif /* util.h */
